import argparse
import logging
import numpy as np
from typing import Any, Callable, List, Optional, Tuple, Dict
from vega_sim.environment.agent import Agent

from vega_sim.scenario.scenario import Scenario
from vega_sim.scenario.common.utils.price_process import random_walk
from vega_sim.environment.environment import MarketEnvironmentWithState
from vega_sim.null_service import VegaServiceNull
from vega_sim.scenario.constants import Network
from vega_sim.scenario.comprehensive_market.agents import (
    MM_WALLET,
    AUCTION1_WALLET,
    AUCTION2_WALLET,
    TERMINATE_WALLET,
    create_agent_wallets,
    INFORMED_WALLET,
)
from vega_sim.scenario.common.agents import (
    MarketManager,
    LiquidityProvider,
    MarketOrderTrader,
    LimitOrderTrader,
    MomentumTrader,
    OpenAuctionPass,
    InformedTrader,
)


class ComprehensiveMarket(Scenario):
    def __init__(
        self,
        num_steps: int = 1000,
        market_decimal: int = 5,
        asset_decimal: int = 5,
        market_position_decimal: int = 2,
        market_name: str = None,
        asset_name: str = None,
        initial_asset_mint: float = 1000000,
        initial_price: Optional[float] = None,
        sigma: float = 1,
        lp_commitamount: float = 200000,
        spread: int = 2,
        block_size: int = 1,
        block_length_seconds: int = 1,
        state_extraction_freq: int = 1,
        step_length_seconds: int = 1,
        opening_auction_trade_amount: float = 1,
        state_extraction_fn: Optional[
            Callable[[VegaServiceNull, List[Agent]], Any]
        ] = None,
        pause_every_n_steps: Optional[int] = None,
        price_process_fn: Optional[Callable[[None], List[float]]] = None,
        market_order_trader_order_intensity: int = 10,
        market_order_trader_order_size: float = 0.1,
        settle_at_end: bool = True,
        limit_order_trader_quantity: int = 5,
        limit_order_trader_submit_bias: float = 0.5,
        limit_order_trader_cancel_bias: float = 0.5,
        limit_order_trader_order_intensity: int = 10,
        limit_order_trader_order_size: float = 0.1,
        limit_order_trader_mean: float = -5,
        limit_order_trader_sigma: float = 0.5,
        limit_order_trader_duration: int = 300,
        limit_order_trader_time_in_force_opts: Optional[dict] = None,
        momentum_trader_order_intensity: int = 10,
        momentum_trader_order_size: float = 0.1,
        momentum_trader_strategies: List[str] = None,
        momentum_trader_strategy_args: List[Dict[str, float]] = None,
        momentum_trader_indicator_thresholds: List[Tuple[float, float]] = None,
        num_lp_agents: int = 3,
        num_mo_agents: int = 5,
        num_lo_agents: int = 20,
        num_momentum_agents: int = 1,
    ):
        self.num_steps = num_steps
        self.market_decimal = market_decimal
        self.asset_decimal = asset_decimal
        self.market_position_decimal = market_position_decimal
        self.initial_price = initial_price
        self.sigma = sigma
        self.spread = spread / 10**self.market_decimal
        self.block_size = block_size
        self.block_length_seconds = block_length_seconds
        self.state_extraction_freq = state_extraction_freq
        self.step_length_seconds = step_length_seconds
        self.state_extraction_fn = state_extraction_fn
        self.pause_every_n_steps = pause_every_n_steps
        self.lp_commitamount = lp_commitamount
        self.initial_asset_mint = initial_asset_mint
        self.market_name = "ETH:USD" if market_name is None else market_name
        self.asset_name = "tDAI" if asset_name is None else asset_name
        self.price_process_fn = price_process_fn
        self.opening_auction_trade_amount = opening_auction_trade_amount
        self.settle_at_end = settle_at_end

        # MarketOrderTraderOptions
        self.market_order_trader_order_intensity = market_order_trader_order_intensity
        self.market_order_trader_order_size = market_order_trader_order_size

        # LimitOrderTrader Options
        self.limit_order_trader_quantity = limit_order_trader_quantity
        self.limit_order_trader_submit_bias = limit_order_trader_submit_bias
        self.limit_order_trader_cancel_bias = limit_order_trader_cancel_bias
        self.limit_order_trader_order_intensity = limit_order_trader_order_intensity
        self.limit_order_trader_order_size = limit_order_trader_order_size
        self.limit_order_trader_mean = limit_order_trader_mean
        self.limit_order_trader_sigma = limit_order_trader_sigma
        self.limit_order_trader_duration = limit_order_trader_duration
        self.limit_order_trader_time_in_force_opts = (
            limit_order_trader_time_in_force_opts
        )

        # MomentumTrader Options
        self.momentum_trader_order_intensity = momentum_trader_order_intensity
        self.momentum_trader_order_size = momentum_trader_order_size
        self.momentum_trader_strategies = (
            momentum_trader_strategies
            if momentum_trader_strategies is not None
            else ["RSI"] * num_momentum_agents
        )
        self.momentum_trader_strategy_args = momentum_trader_strategy_args
        self.momentum_trader_indicator_thresholds = (
            momentum_trader_indicator_thresholds
            if momentum_trader_indicator_thresholds is not None
            else [(70, 30)] * num_momentum_agents
        )

        # Agent options
        self.lp_wallets = create_agent_wallets(n=num_lp_agents, prefix="lp_agent_")
        self.mo_wallets = create_agent_wallets(n=num_mo_agents, prefix="lo_agent_")
        self.lo_wallets = create_agent_wallets(n=num_lo_agents, prefix="mo_agent_")
        self.momentum_wallets = create_agent_wallets(
            n=num_momentum_agents, prefix="momentum_agent_"
        )

    def _generate_price_process(
        self,
        random_state: Optional[np.random.RandomState] = None,
    ):
        return random_walk(
            num_steps=self.num_steps,
            sigma=self.sigma,
            starting_price=self.initial_price,
            random_state=random_state,
        )

    def set_up_background_market(
        self,
        vega: VegaServiceNull,
        tag: str = "",
        random_state: Optional[np.random.RandomState] = None,
    ) -> MarketEnvironmentWithState:
        # Set up market name and settlement asset
        market_name = self.market_name + f"_{tag}"
        asset_name = self.asset_name + f"_{tag}"

        price_process = (
            self.price_process_fn()
            if self.price_process_fn is not None
            else self._generate_price_process(random_state=random_state)
        )

        mm_agent = MarketManager(
            wallet_name=MM_WALLET.name,
            wallet_pass=MM_WALLET.passphrase,
            terminate_wallet_name=TERMINATE_WALLET.name,
            terminate_wallet_pass=TERMINATE_WALLET.passphrase,
            asset_decimal=self.asset_decimal,
            market_decimal=self.market_decimal,
            market_position_decimal=self.market_position_decimal,
            market_name=market_name,
            asset_name=asset_name,
            commitment_amount=self.lp_commitamount,
            tag=str(tag),
            settlement_price=price_process[-1] if self.settle_at_end else None,
        )

        lp_agents = [
            LiquidityProvider(
                wallet_name=self.lp_wallets[i].name,
                wallet_pass=self.lp_wallets[i].passphrase,
                initial_asset_mint=self.initial_asset_mint,
                market_name=market_name,
                asset_name=asset_name,
                tag=str(tag),
                commitment_amount=self.lp_commitamount,
                fee=0.001,
                offset=self.spread * (i + 1),
            )
            for i in range(len(self.lp_wallets))
        ]

        mo_agents = [
            MarketOrderTrader(
                wallet_name=self.mo_wallets[i].name,
                wallet_pass=self.mo_wallets[i].passphrase,
                initial_asset_mint=self.initial_asset_mint,
                market_name=market_name,
                asset_name=asset_name,
                tag=str(tag),
                buy_intensity=self.market_order_trader_order_intensity,
                sell_intensity=self.market_order_trader_order_intensity,
                base_order_size=self.market_order_trader_order_size,
                random_state=random_state,
            )
            for i in range(len(self.mo_wallets))
        ]

        lo_agents = [
            LimitOrderTrader(
                wallet_name=self.lo_wallets[i].name,
                wallet_pass=self.lo_wallets[i].passphrase,
                initial_asset_mint=self.initial_asset_mint,
                market_name=market_name,
                asset_name=asset_name,
                tag=str(tag),
                spread=self.spread,
                price_process=price_process,
                buy_intensity=self.limit_order_trader_order_intensity,
                sell_intensity=self.limit_order_trader_order_intensity,
                buy_volume=self.limit_order_trader_order_size,
                sell_volume=self.limit_order_trader_order_size,
                submit_bias=self.limit_order_trader_submit_bias,
                cancel_bias=self.limit_order_trader_cancel_bias,
                duration=self.limit_order_trader_duration,
                time_in_force_opts=self.limit_order_trader_time_in_force_opts,
                mean=self.limit_order_trader_mean,
                sigma=self.limit_order_trader_sigma,
                random_state=random_state,
            )
            for i in range(len(self.lo_wallets))
        ]

        momentum_agents = [
            MomentumTrader(
                wallet_name=self.momentum_wallets[i].name,
                wallet_pass=self.momentum_wallets[i].passphrase,
                market_name=market_name,
                asset_name=asset_name,
                initial_asset_mint=self.initial_asset_mint,
                order_intensity=self.momentum_trader_order_intensity,
                base_order_size=self.momentum_trader_order_size,
                momentum_strategy=self.momentum_trader_strategies[i],
                momentum_strategy_args=self.momentum_trader_strategy_args[i]
                if self.momentum_trader_strategy_args is not None
                else self.momentum_trader_strategy_args,
                indicator_threshold=self.momentum_trader_indicator_thresholds[i],
                send_limit_order=True,
                offset_levels=20,
                tag=str(tag),
            )
            for i in range(len(self.momentum_wallets))
        ]

        auctionpass1 = OpenAuctionPass(
            wallet_name=AUCTION1_WALLET.name,
            wallet_pass=AUCTION1_WALLET.passphrase,
            side="SIDE_BUY",
            initial_asset_mint=self.initial_asset_mint,
            initial_price=self.initial_price
            if self.initial_price is not None
            else price_process[0],
            market_name=market_name,
            asset_name=asset_name,
            opening_auction_trade_amount=self.opening_auction_trade_amount,
            tag=str(tag),
        )

        auctionpass2 = OpenAuctionPass(
            wallet_name=AUCTION2_WALLET.name,
            wallet_pass=AUCTION2_WALLET.passphrase,
            side="SIDE_SELL",
            initial_asset_mint=self.initial_asset_mint,
            initial_price=self.initial_price
            if self.initial_price is not None
            else price_process[0],
            market_name=market_name,
            asset_name=asset_name,
            opening_auction_trade_amount=self.opening_auction_trade_amount,
            tag=str(tag),
        )

        env = MarketEnvironmentWithState(
            agents=[
                mm_agent,
                auctionpass1,
                auctionpass2,
            ]
            + lp_agents
            + mo_agents
            + lo_agents
            + momentum_agents,
            n_steps=self.num_steps,
            transactions_per_block=self.block_size,
            vega_service=vega,
            state_extraction_freq=self.state_extraction_freq,
            step_length_seconds=self.step_length_seconds,
            block_length_seconds=self.block_length_seconds,
            state_extraction_fn=self.state_extraction_fn,
            pause_every_n_steps=self.pause_every_n_steps,
        )

        info_trader = InformedTrader(
            wallet_name=INFORMED_WALLET.name,
            wallet_pass=INFORMED_WALLET.passphrase,
            price_process=price_process,
            market_name=market_name,
            asset_name=asset_name,
            initial_asset_mint=self.initial_asset_mint,
            proportion_taken=self.proportion_taken,
            tag=str(tag),
        )
        return env

    def run_iteration(
        self,
        vega: VegaServiceNull,
        network: Optional[Network] = None,
        pause_at_completion: bool = False,
        run_with_console: bool = False,
        random_state: Optional[np.random.RandomState] = None,
    ):
        env = self.set_up_background_market(
            vega=vega, tag=str(0), random_state=random_state
        )
        result = env.run(
            pause_at_completion=pause_at_completion,
            run_with_console=run_with_console,
        )
        return result


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--debug", action="store_true")
    args = parser.parse_args()

    logging.basicConfig(level=logging.INFO if not args.debug else logging.DEBUG)

    step_length = 60 * 60

    scenario = ComprehensiveMarket(num_steps=200)

    with VegaServiceNull(
        warn_on_raw_data_access=False,
        run_with_console=True,
        seconds_per_block=40,  # Heuristic
    ) as vega:
        scenario.run_iteration(
            vega=vega,
            pause_at_completion=True,
        )



import logging
from collections import namedtuple
from dataclasses import dataclass
from typing import DefaultDict, Dict, Iterable, List, Optional, Tuple, Any

import vega_sim.api.data_raw as data_raw
import vega_sim.grpc.client as vac
import vega_sim.proto.data_node.api.v2 as data_node_protos_v2
import vega_sim.proto.vega as vega_protos
from vega_sim.api.helpers import num_from_padded_int


class MissingAssetError(Exception):
    pass


class MissingMarketError(Exception):
    pass


logger = logging.Logger(__name__)


AccountData = namedtuple("AccountData", ["general", "margin", "bond"])
OrderBook = namedtuple("OrderBook", ["bids", "asks"])
PriceLevel = namedtuple("PriceLevel", ["price", "number_of_orders", "volume"])
Order = namedtuple(
    "Order",
    [
        "price",
        "size",
        "id",
        "reference",
        "side",
        "status",
        "remaining",
        "time_in_force",
        "order_type",
        "created_at",
        "expires_at",
        "party_id",
        "market_id",
        "updated_at",
        "version",
    ],
)
Position = namedtuple(
    "Position",
    [
        "party_id",
        "market_id",
        "open_volume",
        "realised_pnl",
        "unrealised_pnl",
        "average_entry_price",
        "updated_at",
    ],
)

MarginLevels = namedtuple(
    "MarginLevels",
    [
        "maintenance_margin",
        "search_level",
        "initial_margin",
        "collateral_release_level",
        "party_id",
        "market_id",
        "asset",
        "timestamp",
    ],
)


@dataclass
class MarketDepth:
    buys: List[PriceLevel]
    sells: List[PriceLevel]


@dataclass
class OrdersBySide:
    bids: List[Order]
    asks: List[Order]


@dataclass
class Fee:
    maker_fee: float
    infrastructure_fee: float
    liquidity_fee: float


@dataclass
class Trade:
    id: str
    market_id: str
    price: float
    size: float
    buyer: str
    seller: str
    aggressor: vega_protos.vega.Side
    buy_order: str
    sell_order: str
    timestamp: int
    trade_type: vega_protos.vega.Trade.Type
    buyer_fee: Fee
    seller_fee: Fee
    buyer_auction_batch: int
    seller_auction_batch: int


def _trade_from_proto(
    trade: vega_protos.vega.Trade,
    price_decimals: int,
    position_decimals: int,
    asset_decimals: int,
) -> Trade:
    return Trade(
        id=trade.id,
        market_id=trade.market_id,
        price=num_from_padded_int(trade.price, price_decimals),
        size=num_from_padded_int(trade.size, position_decimals),
        buyer=trade.buyer,
        seller=trade.seller,
        aggressor=trade.aggressor,
        buy_order=trade.buy_order,
        sell_order=trade.sell_order,
        timestamp=trade.timestamp,
        trade_type=trade.type,
        buyer_fee=Fee(
            maker_fee=num_from_padded_int(trade.buyer_fee.maker_fee, asset_decimals),
            infrastructure_fee=num_from_padded_int(
                trade.buyer_fee.infrastructure_fee, asset_decimals
            ),
            liquidity_fee=num_from_padded_int(
                trade.buyer_fee.liquidity_fee, asset_decimals
            ),
        ),
        seller_fee=Fee(
            maker_fee=num_from_padded_int(trade.seller_fee.maker_fee, asset_decimals),
            infrastructure_fee=num_from_padded_int(
                trade.seller_fee.infrastructure_fee, asset_decimals
            ),
            liquidity_fee=num_from_padded_int(
                trade.seller_fee.liquidity_fee, asset_decimals
            ),
        ),
        buyer_auction_batch=trade.buyer_auction_batch,
        seller_auction_batch=trade.seller_auction_batch,
    )


def _margin_level_from_proto(
    margin_level: vega_protos.vega.MarginLevels, asset_decimals: int
) -> MarginLevels:
    return MarginLevels(
        maintenance_margin=num_from_padded_int(
            margin_level.maintenance_margin, asset_decimals
        ),
        search_level=num_from_padded_int(margin_level.search_level, asset_decimals),
        initial_margin=num_from_padded_int(margin_level.initial_margin, asset_decimals),
        collateral_release_level=num_from_padded_int(
            margin_level.collateral_release_level, asset_decimals
        ),
        party_id=margin_level.party_id,
        market_id=margin_level.market_id,
        asset=margin_level.asset,
        timestamp=margin_level.timestamp,
    )


def _order_from_proto(
    order: vega_protos.vega.Order, price_decimals: int, position_decimals: int
) -> Order:
    return Order(
        id=order.id,
        price=num_from_padded_int(order.price, price_decimals),
        size=num_from_padded_int(order.size, position_decimals),
        reference=order.reference,
        side=order.side,
        status=order.status,
        remaining=num_from_padded_int(order.remaining, position_decimals),
        time_in_force=order.time_in_force,
        order_type=order.type,
        created_at=order.created_at,
        expires_at=order.expires_at,
        party_id=order.party_id,
        updated_at=order.updated_at,
        version=order.version,
        market_id=order.market_id,
    )


def _position_from_proto(
    position: vega_protos.vega.Position,
    position_decimals: int,
    asset_decimals: int,
    price_decimals: int,
) -> Position:
    return Position(
        party_id=position.party_id,
        market_id=position.market_id,
        open_volume=num_from_padded_int(position.open_volume, position_decimals),
        realised_pnl=num_from_padded_int(position.realised_pnl, asset_decimals),
        unrealised_pnl=num_from_padded_int(position.unrealised_pnl, asset_decimals),
        average_entry_price=num_from_padded_int(
            position.average_entry_price, price_decimals
        ),
        updated_at=position.updated_at,
    )


def positions_by_market(
    pub_key: str,
    market_id: str,
    position_decimals: int,
    asset_decimals: int,
    price_decimals: int,
    data_client: vac.VegaTradingDataClientV2,
) -> List[Position]:
    """Output positions of a party."""
    return [
        _position_from_proto(
            pos,
            position_decimals=position_decimals,
            asset_decimals=asset_decimals,
            price_decimals=price_decimals,
        )
        for pos in data_raw.positions_by_market(
            pub_key=pub_key, market_id=market_id, data_client=data_client
        )
    ]


def party_account(
    pub_key: str,
    asset_id: str,
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
    asset_dp: Optional[int] = None,
) -> AccountData:
    """Output money in general accounts/margin accounts/bond accounts (if exists)
    of a party."""
    accounts = data_raw.party_accounts(
        data_client=data_client,
        party_id=pub_key,
        asset_id=asset_id,
    )

    asset_dp = (
        asset_dp if asset_dp is not None else asset_decimals(asset_id, data_client)
    )

    general, margin, bond = 0, 0, 0  # np.nan, np.nan, np.nan
    for account in accounts:
        if (
            account.market_id
            and account.market_id != "!"
            and account.market_id != market_id
        ):
            # The 'general' account type has no market ID, so we have to pull
            # all markets then filter down here
            continue
        if account.type == vega_protos.vega.ACCOUNT_TYPE_GENERAL:
            general = num_from_padded_int(float(account.balance), asset_dp)
        if account.type == vega_protos.vega.ACCOUNT_TYPE_MARGIN:
            margin = num_from_padded_int(float(account.balance), asset_dp)
        if account.type == vega_protos.vega.ACCOUNT_TYPE_BOND:
            bond = num_from_padded_int(float(account.balance), asset_dp)

    return AccountData(general, margin, bond)


def find_market_id(
    name: str, data_client: vac.VegaTradingDataClientV2, raise_on_missing: bool = False
) -> str:
    """Looks up the Market ID of a given market name

    Args:
        symbol:
            str, The symbol of the asset to look up
        data_client:
            VegaTradingDataClientV2, the gRPC data client
        raise_on_missing:
            bool, whether to raise an Error or silently return
                if the asset does not exist

    Returns:
        str, the ID of the asset
    """
    markets = data_raw.all_markets(data_client=data_client)

    acceptable_states = [
        vega_protos.markets.Market.STATE_PENDING,
        vega_protos.markets.Market.STATE_ACTIVE,
        vega_protos.markets.Market.STATE_SUSPENDED,
    ]

    market_ids = {}

    for market in markets:

        if market.tradable_instrument.instrument.name == name:
            if market.state in acceptable_states:
                market_ids[market.id] = market.market_timestamps.pending

    if len(market_ids) > 0:
        return max(market_ids, key=market_ids.get)

    if raise_on_missing:
        raise MissingMarketError(
            f"{name} market not found on specified Vega network, "
            + "please propose and create this market first"
        )


def find_asset_id(
    symbol: str,
    data_client: vac.VegaTradingDataClientV2,
    raise_on_missing: bool = False,
) -> str:
    """Looks up the Asset ID of a given asset name

    Args:
        symbol:
            str, The symbol of the asset to look up
        data_client:
            VegaTradingDataClientV2, the gRPC data client
        raise_on_missing:
            bool, whether to raise an Error or silently return
                if the asset does not exist

    Returns:
        str, the ID of the asset
    """
    assets = data_raw.list_assets(data_client=data_client)
    # Find settlement asset
    for asset in assets:
        if asset.details.symbol == symbol:
            return asset.id
    if raise_on_missing:
        raise MissingAssetError(
            f"{symbol} asset not found on specified Vega network, "
            + "please propose and create this asset first"
        )


def market_price_decimals(
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
) -> int:
    """Returns the number of decimal places a specified market uses for price units.

    Args:
        market_id:
            str, The ID of the market requested
        data_client:
            VegaTradingDataClientV2, an instantiated gRPC data client

    Returns:
        int, The number of decimal places the market uses
    """
    return data_raw.market_info(
        market_id=market_id, data_client=data_client
    ).decimal_places


def market_position_decimals(
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
) -> int:
    """Returns the number of decimal places a specified market uses for position units.

    Args:
        market_id:
            str, The ID of the market requested
        data_client:
            VegaTradingDataClientV2, an instantiated gRPC data client

    Returns:
        int, The number of decimal places the market uses for positions
    """
    return data_raw.market_info(
        market_id=market_id, data_client=data_client
    ).position_decimal_places


def asset_decimals(
    asset_id: str,
    data_client: vac.VegaTradingDataClientV2,
) -> int:
    """Returns the number of decimal places a specified asset uses for price.

    Args:
        asset_id:
            str, The ID of the asset requested
        data_client:
            VegaTradingDataClientV2, an instantiated gRPC data client

    Returns:
        int, The number of decimal places the asset uses
    """
    return data_raw.asset_info(
        asset_id=asset_id, data_client=data_client
    ).details.decimals


def best_prices(
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
    price_decimals: Optional[int] = None,
) -> Tuple[float, float]:
    """
    Output the best static bid price and best static ask price in current market.
    """
    mkt_data = data_raw.market_data(market_id=market_id, data_client=data_client)
    mkt_price_dp = (
        price_decimals
        if price_decimals is not None
        else market_price_decimals(market_id=market_id, data_client=data_client)
    )

    return (
        num_from_padded_int(mkt_data.best_static_bid_price, mkt_price_dp),
        num_from_padded_int(mkt_data.best_static_offer_price, mkt_price_dp),
    )


def open_orders_by_market(
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
    price_decimals: Optional[int] = None,
    position_decimals: Optional[int] = None,
) -> OrdersBySide:
    """
    Output all active limit orders in current market.

    Args:
        market_id:
            str, ID for the market to load
        data_client:
            VegaTradingDataClientV2, instantiated gRPC client

    Returns:
        OrdersBySide, Live orders segregated by side
    """
    bids = []
    asks = []
    orders = all_orders(
        market_id=market_id,
        data_client=data_client,
        price_decimals=price_decimals,
        position_decimals=position_decimals,
        open_only=True,
    )
    for order in orders:
        bids.append(order) if order.side == vega_protos.vega.SIDE_BUY else asks.append(
            order
        )

    return OrdersBySide(bids, asks)


def list_orders(
    data_client: vac.VegaTradingDataClientV2,
    market_id: Optional[str] = None,
    party_id: Optional[str] = None,
    reference: Optional[str] = None,
    price_decimals: Optional[int] = None,
    position_decimals: Optional[int] = None,
    live_only: Optional[bool] = True,
) -> List[Order]:
    """Return a list of converted orders for the specified market, party, and reference.

    Args:
        data_client (vac.VegaTradingDataClientV2):
            An instantiated gRPC trading_data_client_v2.
        market_id (str):
            Id of market to return orders for.
        party_id (str):
            Id of party to return orders for.
        reference (str):
            References to return orders for.
        price_decimals (Optional[int], optional):
            Market price decimal places. Defaults to None.
        position_decimals (Optional[int], optional):
            Market position decimal places. Defaults to None.
        live_only (Optional[bool], optional):
            Whether to only return live orders. Defaults to True.

    Returns:
        List[Order]:
            A list of converted orders.
    """
    orders = data_raw.list_orders(
        data_client=data_client,
        party_id=party_id,
        market_id=market_id,
        reference=reference,
        live_only=live_only,
    )

    mkt_price_dp = (
        DefaultDict(lambda: price_decimals) if price_decimals is not None else {}
    )
    mkt_pos_dp = (
        DefaultDict(lambda: position_decimals) if position_decimals is not None else {}
    )

    output_orders = []
    for order in orders:
        if price_decimals is None and order.market_id not in mkt_price_dp:
            mkt_pos_dp[order.market_id] = market_position_decimals(
                market_id=order.market_id, data_client=data_client
            )
            mkt_price_dp[order.market_id] = market_price_decimals(
                market_id=order.market_id, data_client=data_client
            )

        converted_order = _order_from_proto(
            order,
            price_decimals=mkt_price_dp[order.market_id],
            position_decimals=mkt_pos_dp[order.market_id],
        )
        output_orders.append(converted_order)
    return output_orders


def all_orders(
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
    price_decimals: Optional[int] = None,
    position_decimals: Optional[int] = None,
    open_only: bool = False,
) -> OrdersBySide:
    """
    Output all active limit orders in current market.

    Args:
        market_id:
            str, ID for the market to load
        data_client:
            VegaTradingDataClientV2, instantiated gRPC client
        open_only:
            bool, default False, whether to only return still
                open orders

    Returns:
        OrdersBySide, Live orders segregated by side
    """
    orders = data_raw.list_orders(
        market_id=market_id,
        data_client=data_client,
    )

    mkt_price_dp = (
        DefaultDict(lambda: price_decimals) if price_decimals is not None else {}
    )
    mkt_pos_dp = (
        DefaultDict(lambda: position_decimals) if position_decimals is not None else {}
    )

    output_orders = []
    for order in orders:
        if price_decimals is None and order.market_id not in mkt_price_dp:
            mkt_pos_dp[order.market_id] = market_position_decimals(
                market_id=order.market_id, data_client=data_client
            )
            mkt_price_dp[order.market_id] = market_price_decimals(
                market_id=order.market_id, data_client=data_client
            )

        if open_only and order.status != vega_protos.vega.Order.Status.STATUS_ACTIVE:
            continue
        converted_order = _order_from_proto(
            order,
            price_decimals=mkt_price_dp[order.market_id],
            position_decimals=mkt_pos_dp[order.market_id],
        )
        output_orders.append(converted_order)
    return output_orders


def order_book_by_market(
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
) -> OrderBook:
    """
    Output state of order book for a given market.
    """

    orders = data_raw.list_orders(
        market_id=market_id,
        data_client=data_client,
    )

    mkt_price_dp = market_price_decimals(market_id=market_id, data_client=data_client)
    mkt_pos_dp = market_position_decimals(market_id=market_id, data_client=data_client)

    bids = {}
    asks = {}

    for order in orders:
        if order.status == vega_protos.vega.Order.Status.STATUS_ACTIVE:
            if order.side == vega_protos.vega.Side.SIDE_BUY:
                price = num_from_padded_int(order.price, mkt_price_dp)
                volume = num_from_padded_int(order.remaining, mkt_pos_dp)
                bids[price] = bids.get(price, 0) + volume
            else:
                price = num_from_padded_int(order.price, mkt_price_dp)
                volume = num_from_padded_int(order.remaining, mkt_pos_dp)
                asks[price] = bids.get(price, 0) + volume

    return OrderBook(bids, asks)


def market_account(
    market_id: str,
    account_type: vega_protos.vega.AccountType,
    data_client: vac.VegaTradingDataClientV2,
) -> float:
    """
    Returns the current asset value in the Market's fee account

    Args:
        market_id:
            str, The ID of the market to check
        account_type:
            vega.AccountType, the account type to check for

    Returns:
        float, the current balance in the market's fee asset
    """
    accounts = data_raw.all_market_accounts(
        asset_id=None, market_id=market_id, data_client=data_client
    )
    acct = accounts[account_type]

    asset_dp = asset_decimals(asset_id=acct.asset, data_client=data_client)
    return num_from_padded_int(
        acct.balance,
        asset_dp,
    )


def market_depth(
    market_id: str,
    data_client: vac.VegaTradingDataClientV2,
    max_depth: Optional[int] = None,
    price_decimals: Optional[int] = None,
    position_decimals: Optional[int] = None,
) -> Optional[MarketDepth]:
    mkt_depth = data_raw.market_depth(
        data_client=data_client,
        market_id=market_id,
        max_depth=max_depth,
    )
    if mkt_depth is None:
        return mkt_depth

    mkt_price_dp = (
        price_decimals
        if price_decimals is not None
        else market_price_decimals(market_id=market_id, data_client=data_client)
    )
    mkt_pos_dp = (
        position_decimals
        if position_decimals is not None
        else market_position_decimals(market_id=market_id, data_client=data_client)
    )

    def _price_level_from_raw(level) -> PriceLevel:
        return PriceLevel(
            price=num_from_padded_int(
                level.price,
                mkt_price_dp,
            ),
            number_of_orders=level.number_of_orders,
            volume=num_from_padded_int(
                level.volume,
                mkt_pos_dp,
            ),
        )

    return MarketDepth(
        buys=[_price_level_from_raw(level) for level in mkt_depth.buy],
        sells=[_price_level_from_raw(level) for level in mkt_depth.sell],
    )


def order_subscription(
    data_client: vac.VegaCoreClient,
    trading_data_client: vac.VegaTradingDataClientV2,
    market_id: Optional[str] = None,
    party_id: Optional[str] = None,
) -> Iterable[Order]:
    """Subscribe to a stream of Order updates from the data-node.
    The stream of orders returned from this function is an iterable which
    does not end and will continue to tick another order update whenever
    one is received.

    Args:
        market_id:
            Optional[str], If provided, only update orders from this market
        party_id:
            Optional[str], If provided, only update orders from this party
    Returns:
        Iterable[Order], Infinite iterable of order updates
    """

    order_stream = data_raw.order_subscription(
        data_client=data_client, market_id=market_id, party_id=party_id
    )

    def _order_gen(
        order_stream: Iterable[vega_protos.api.v1.core.ObserveEventBusResponse],
    ) -> Iterable[Order]:
        mkt_pos_dp = {}
        mkt_price_dp = {}
        try:
            for order_list in order_stream:
                for bus_event in order_list.events:
                    order = bus_event.order
                    if order.market_id not in mkt_pos_dp:
                        mkt_pos_dp[order.market_id] = market_position_decimals(
                            market_id=order.market_id, data_client=trading_data_client
                        )
                        mkt_price_dp[order.market_id] = market_price_decimals(
                            market_id=order.market_id, data_client=trading_data_client
                        )
                    yield _order_from_proto(
                        order,
                        mkt_price_dp[order.market_id],
                        mkt_pos_dp[order.market_id],
                    )
        except Exception as _:
            logger.info("Order subscription closed")
            return

    return _order_gen(order_stream=order_stream)


def has_liquidity_provision(
    data_client: vac.VegaTradingDataClientV2,
    market_id: str,
    party_id: str,
) -> bool:
    lip = data_raw.liquidity_provisions(
        data_client=data_client, market_id=market_id, party_id=party_id
    )
    return (
        lip
        and len(lip) > 0
        and lip[0].status
        in [
            vega_protos.vega.LiquidityProvision.Status.STATUS_ACTIVE,
            vega_protos.vega.LiquidityProvision.Status.STATUS_UNDEPLOYED,
            vega_protos.vega.LiquidityProvision.Status.STATUS_PENDING,
        ]
    )


def margin_levels(
    data_client: vac.VegaTradingDataClientV2,
    party_id: str,
    market_id: Optional[str] = None,
) -> List[MarginLevels]:
    asset_dp = {}
    margins = data_raw.margin_levels(
        data_client=data_client, party_id=party_id, market_id=market_id
    )
    res_margins = []
    for margin in margins:
        if margin.asset not in asset_dp:
            asset_dp[margin.asset] = asset_decimals(
                asset_id=margin.asset, data_client=data_client
            )
        res_margins.append(
            _margin_level_from_proto(margin, asset_decimals=asset_dp[margin.asset])
        )
    return res_margins


def get_trades(
    data_client: vac.VegaTradingDataClientV2,
    market_id: str,
    party_id: Optional[str] = None,
    order_id: Optional[str] = None,
    market_price_decimals_map: Optional[Dict[str, int]] = None,
    market_position_decimals_map: Optional[Dict[str, int]] = None,
    market_asset_decimals_map: Optional[Dict[str, int]] = None,
) -> List[Trade]:
    base_trades = data_raw.get_trades(
        data_client=data_client,
        party_id=party_id,
        market_id=market_id,
        order_id=order_id,
    )
    market_price_decimals_map = market_price_decimals_map or {}
    market_position_decimals_map = market_position_decimals_map or {}
    market_asset_decimals_map = market_asset_decimals_map or {}

    res_trades = []
    for trade in base_trades:
        if trade.market_id not in market_price_decimals_map:
            market_price_decimals_map[trade.market_id] = market_price_decimals(
                market_id=trade.market_id, data_client=data_client
            )
        if trade.market_id not in market_position_decimals_map:
            market_position_decimals_map[trade.market_id] = market_position_decimals(
                market_id=trade.market_id, data_client=data_client
            )
        if trade.market_id not in market_asset_decimals_map:
            market_asset_decimals_map[trade.market_id] = asset_decimals(
                asset_id=data_raw.market_info(
                    market_id=market_id, data_client=data_client
                ).tradable_instrument.instrument.future.settlement_asset,
                data_client=data_client,
            )
        res_trades.append(
            _trade_from_proto(
                trade,
                price_decimals=market_price_decimals_map[trade.market_id],
                position_decimals=market_position_decimals_map[trade.market_id],
                asset_decimals=market_asset_decimals_map[trade.market_id],
            )
        )
    return res_trades


def ping(data_client: vac.VegaTradingDataClientV2):
    return data_client.Ping(data_node_protos_v2.trading_data.PingRequest())



from . import commands_pb2 as commands
from . import commands_pb2_grpc as commands_grpc
from . import data_pb2 as data
from . import data_pb2_grpc as data_grpc
from . import signature_pb2 as signature
from . import signature_pb2_grpc as signature_grpc
from . import transaction_pb2 as transaction
from . import transaction_pb2_grpc as transaction_grpc
from . import validator_commands_pb2 as validator_commands
from . import validator_commands_pb2_grpc as validator_commands_grpc

__all__ = [
    "commands",
    "commands_grpc",
    "data",
    "data_grpc",
    "signature",
    "signature_grpc",
    "transaction",
    "transaction_grpc",
    "validator_commands",
    "validator_commands_grpc",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data-node/api/v1/trading_data.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ....vega import vega_pb2 as vega_dot_vega__pb2
from ....vega import markets_pb2 as vega_dot_markets__pb2
from ....vega import governance_pb2 as vega_dot_governance__pb2
from ....vega import assets_pb2 as vega_dot_assets__pb2
from ....vega import oracle_pb2 as vega_dot_oracle__pb2
from ....vega.events.v1 import events_pb2 as vega_dot_events_dot_v1_dot_events__pb2
from ....vega.commands.v1 import (
    validator_commands_pb2 as vega_dot_commands_dot_v1_dot_validator__commands__pb2,
)


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n#data-node/api/v1/trading_data.proto\x12\x0f\x64\x61tanode.api.v1\x1a\x0fvega/vega.proto\x1a\x12vega/markets.proto\x1a\x15vega/governance.proto\x1a\x11vega/assets.proto\x1a\x11vega/oracle.proto\x1a\x1bvega/events/v1/events.proto\x1a)vega/commands/v1/validator_commands.proto"X\n\x10TransfersRequest\x12\x16\n\x06pubkey\x18\x01 \x01(\tR\x06pubkey\x12\x17\n\x07is_from\x18\x02 \x01(\x08R\x06isFrom\x12\x13\n\x05is_to\x18\x03 \x01(\x08R\x04isTo"K\n\x11TransfersResponse\x12\x36\n\ttransfers\x18\x01 \x03(\x0b\x32\x18.vega.events.v1.TransferR\ttransfers"\x9d\x01\n\x12\x44\x65legationsRequest\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x12\x17\n\x07node_id\x18\x02 \x01(\tR\x06nodeId\x12\x1b\n\tepoch_seq\x18\x03 \x01(\tR\x08\x65pochSeq\x12;\n\npagination\x18\x04 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"I\n\x13\x44\x65legationsResponse\x12\x32\n\x0b\x64\x65legations\x18\x01 \x03(\x0b\x32\x10.vega.DelegationR\x0b\x64\x65legations"J\n\x19ObserveDelegationsRequest\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party\x12\x17\n\x07node_id\x18\x02 \x01(\tR\x06nodeId"N\n\x1aObserveDelegationsResponse\x12\x30\n\ndelegation\x18\x01 \x01(\x0b\x32\x10.vega.DelegationR\ndelegation")\n\x11PartyStakeRequest\x12\x14\n\x05party\x18\x01 \x01(\tR\x05party"\x91\x01\n\x12PartyStakeResponse\x12\x36\n\x17\x63urrent_stake_available\x18\x01 \x01(\tR\x15\x63urrentStakeAvailable\x12\x43\n\x0estake_linkings\x18\x02 \x03(\x0b\x32\x1c.vega.events.v1.StakeLinkingR\rstakeLinkings"\x14\n\x12GetNodeDataRequest"B\n\x13GetNodeDataResponse\x12+\n\tnode_data\x18\x01 \x01(\x0b\x32\x0e.vega.NodeDataR\x08nodeData"\x11\n\x0fGetNodesRequest"4\n\x10GetNodesResponse\x12 \n\x05nodes\x18\x01 \x03(\x0b\x32\n.vega.NodeR\x05nodes"$\n\x12GetNodeByIDRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"5\n\x13GetNodeByIDResponse\x12\x1e\n\x04node\x18\x01 \x01(\x0b\x32\n.vega.NodeR\x04node"!\n\x0fGetEpochRequest\x12\x0e\n\x02id\x18\x01 \x01(\x04R\x02id"5\n\x10GetEpochResponse\x12!\n\x05\x65poch\x18\x01 \x01(\x0b\x32\x0b.vega.EpochR\x05\x65poch"\x89\x01\n\x0bKeyRotation\x12\x17\n\x07node_id\x18\x01 \x01(\tR\x06nodeId\x12\x1e\n\x0bold_pub_key\x18\x02 \x01(\tR\toldPubKey\x12\x1e\n\x0bnew_pub_key\x18\x03 \x01(\tR\tnewPubKey\x12!\n\x0c\x62lock_height\x18\x04 \x01(\x04R\x0b\x62lockHeight"\x18\n\x16GetKeyRotationsRequest"U\n\x17GetKeyRotationsResponse\x12:\n\trotations\x18\x01 \x03(\x0b\x32\x1c.datanode.api.v1.KeyRotationR\trotations"7\n\x1cGetKeyRotationsByNodeRequest\x12\x17\n\x07node_id\x18\x01 \x01(\tR\x06nodeId"[\n\x1dGetKeyRotationsByNodeResponse\x12:\n\trotations\x18\x01 \x03(\x0b\x32\x1c.datanode.api.v1.KeyRotationR\trotations"\x0f\n\rAssetsRequest"5\n\x0e\x41ssetsResponse\x12#\n\x06\x61ssets\x18\x01 \x03(\x0b\x32\x0b.vega.AssetR\x06\x61ssets""\n\x10\x41ssetByIDRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"6\n\x11\x41ssetByIDResponse\x12!\n\x05\x61sset\x18\x01 \x01(\x0b\x32\x0b.vega.AssetR\x05\x61sset"3\n!GetNodeSignaturesAggregateRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"e\n"GetNodeSignaturesAggregateResponse\x12?\n\nsignatures\x18\x01 \x03(\x0b\x32\x1f.vega.commands.v1.NodeSignatureR\nsignatures"l\n\x13GetProposalsRequest\x12\x41\n\x0fselect_in_state\x18\x01 \x01(\x0e\x32\x14.vega.Proposal.StateH\x00R\rselectInState\x88\x01\x01\x42\x12\n\x10_select_in_state"@\n\x14GetProposalsResponse\x12(\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"\x8e\x01\n\x1aGetProposalsByPartyRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x41\n\x0fselect_in_state\x18\x02 \x01(\x0e\x32\x14.vega.Proposal.StateH\x00R\rselectInState\x88\x01\x01\x42\x12\n\x10_select_in_state"G\n\x1bGetProposalsByPartyResponse\x12(\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"3\n\x16GetVotesByPartyRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId";\n\x17GetVotesByPartyResponse\x12 \n\x05votes\x18\x01 \x03(\x0b\x32\n.vega.VoteR\x05votes"u\n\x1cGetNewMarketProposalsRequest\x12\x41\n\x0fselect_in_state\x18\x01 \x01(\x0e\x32\x14.vega.Proposal.StateH\x00R\rselectInState\x88\x01\x01\x42\x12\n\x10_select_in_state"I\n\x1dGetNewMarketProposalsResponse\x12(\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"\x95\x01\n\x1fGetUpdateMarketProposalsRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\x41\n\x0fselect_in_state\x18\x02 \x01(\x0e\x32\x14.vega.Proposal.StateH\x00R\rselectInState\x88\x01\x01\x42\x12\n\x10_select_in_state"L\n GetUpdateMarketProposalsResponse\x12(\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"}\n$GetNetworkParametersProposalsRequest\x12\x41\n\x0fselect_in_state\x18\x01 \x01(\x0e\x32\x14.vega.Proposal.StateH\x00R\rselectInState\x88\x01\x01\x42\x12\n\x10_select_in_state"Q\n%GetNetworkParametersProposalsResponse\x12(\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"w\n\x1eGetNewFreeformProposalsRequest\x12\x41\n\x0fselect_in_state\x18\x01 \x01(\x0e\x32\x14.vega.Proposal.StateH\x00R\rselectInState\x88\x01\x01\x42\x12\n\x10_select_in_state"K\n\x1fGetNewFreeformProposalsResponse\x12(\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"t\n\x1bGetNewAssetProposalsRequest\x12\x41\n\x0fselect_in_state\x18\x01 \x01(\x0e\x32\x14.vega.Proposal.StateH\x00R\rselectInState\x88\x01\x01\x42\x12\n\x10_select_in_state"H\n\x1cGetNewAssetProposalsResponse\x12(\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"9\n\x16GetProposalByIDRequest\x12\x1f\n\x0bproposal_id\x18\x01 \x01(\tR\nproposalId"C\n\x17GetProposalByIDResponse\x12(\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"=\n\x1dGetProposalByReferenceRequest\x12\x1c\n\treference\x18\x01 \x01(\tR\treference"J\n\x1eGetProposalByReferenceResponse\x12(\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"\x1a\n\x18ObserveGovernanceRequest"E\n\x19ObserveGovernanceResponse\x12(\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta"9\n\x1cObservePartyProposalsRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId"I\n\x1dObservePartyProposalsResponse\x12(\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x14.vega.GovernanceDataR\x04\x64\x61ta">\n\x1bObserveProposalVotesRequest\x12\x1f\n\x0bproposal_id\x18\x01 \x01(\tR\nproposalId">\n\x1cObserveProposalVotesResponse\x12\x1e\n\x04vote\x18\x01 \x01(\x0b\x32\n.vega.VoteR\x04vote"5\n\x18ObservePartyVotesRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId";\n\x19ObservePartyVotesResponse\x12\x1e\n\x04vote\x18\x01 \x01(\x0b\x32\n.vega.VoteR\x04vote"V\n\x1cMarginLevelsSubscribeRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId"X\n\x1dMarginLevelsSubscribeResponse\x12\x37\n\rmargin_levels\x18\x01 \x01(\x0b\x32\x12.vega.MarginLevelsR\x0cmarginLevels"M\n\x13MarginLevelsRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId"O\n\x14MarginLevelsResponse\x12\x37\n\rmargin_levels\x18\x01 \x03(\x0b\x32\x12.vega.MarginLevelsR\x0cmarginLevels":\n\x1bMarketsDataSubscribeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId"Q\n\x1cMarketsDataSubscribeResponse\x12\x31\n\x0bmarket_data\x18\x01 \x01(\x0b\x32\x10.vega.MarketDataR\nmarketData"4\n\x15MarketDataByIDRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId"K\n\x16MarketDataByIDResponse\x12\x31\n\x0bmarket_data\x18\x01 \x01(\x0b\x32\x10.vega.MarketDataR\nmarketData"\x14\n\x12MarketsDataRequest"J\n\x13MarketsDataResponse\x12\x33\n\x0cmarkets_data\x18\x01 \x03(\x0b\x32\x10.vega.MarketDataR\x0bmarketsData"/\n\x10LastTradeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId"6\n\x11LastTradeResponse\x12!\n\x05trade\x18\x01 \x01(\x0b\x32\x0b.vega.TradeR\x05trade"0\n\x11MarketByIDRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId":\n\x12MarketByIDResponse\x12$\n\x06market\x18\x01 \x01(\x0b\x32\x0c.vega.MarketR\x06market"-\n\x10PartyByIDRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId"6\n\x11PartyByIDResponse\x12!\n\x05party\x18\x01 \x01(\x0b\x32\x0b.vega.PartyR\x05party"\x10\n\x0ePartiesRequest"8\n\x0fPartiesResponse\x12%\n\x07parties\x18\x01 \x03(\x0b\x32\x0b.vega.PartyR\x07parties"\x8b\x01\n\x14TradesByPartyRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId\x12;\n\npagination\x18\x03 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"<\n\x15TradesByPartyResponse\x12#\n\x06trades\x18\x01 \x03(\x0b\x32\x0b.vega.TradeR\x06trades"1\n\x14TradesByOrderRequest\x12\x19\n\x08order_id\x18\x01 \x01(\tR\x07orderId"<\n\x15TradesByOrderResponse\x12#\n\x06trades\x18\x01 \x03(\x0b\x32\x0b.vega.TradeR\x06trades"\x8f\x01\n\x18\x41\x63\x63ountsSubscribeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\x19\n\x08party_id\x18\x02 \x01(\tR\x07partyId\x12\x14\n\x05\x61sset\x18\x03 \x01(\tR\x05\x61sset\x12%\n\x04type\x18\x04 \x01(\x0e\x32\x11.vega.AccountTypeR\x04type"D\n\x19\x41\x63\x63ountsSubscribeResponse\x12\'\n\x07\x61\x63\x63ount\x18\x01 \x01(\x0b\x32\r.vega.AccountR\x07\x61\x63\x63ount"P\n\x16OrdersSubscribeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\x19\n\x08party_id\x18\x02 \x01(\tR\x07partyId"P\n\x16TradesSubscribeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\x19\n\x08party_id\x18\x02 \x01(\tR\x07partyId"b\n\x17\x43\x61ndlesSubscribeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12*\n\x08interval\x18\x02 \x01(\x0e\x32\x0e.vega.IntervalR\x08interval"@\n\x18\x43\x61ndlesSubscribeResponse\x12$\n\x06\x63\x61ndle\x18\x01 \x01(\x0b\x32\x0c.vega.CandleR\x06\x63\x61ndle":\n\x1bMarketDepthSubscribeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId"T\n\x1cMarketDepthSubscribeResponse\x12\x34\n\x0cmarket_depth\x18\x01 \x01(\x0b\x32\x11.vega.MarketDepthR\x0bmarketDepth"A\n"MarketDepthUpdatesSubscribeRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId"V\n#MarketDepthUpdatesSubscribeResponse\x12/\n\x06update\x18\x01 \x01(\x0b\x32\x17.vega.MarketDepthUpdateR\x06update"S\n\x19PositionsSubscribeRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId"H\n\x1aPositionsSubscribeResponse\x12*\n\x08position\x18\x01 \x01(\x0b\x32\x0e.vega.PositionR\x08position"q\n\x15OrdersByMarketRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12;\n\npagination\x18\x02 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"=\n\x16OrdersByMarketResponse\x12#\n\x06orders\x18\x01 \x03(\x0b\x32\x0b.vega.OrderR\x06orders"n\n\x14OrdersByPartyRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12;\n\npagination\x18\x02 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"<\n\x15OrdersByPartyResponse\x12#\n\x06orders\x18\x01 \x03(\x0b\x32\x0b.vega.OrderR\x06orders"S\n\x19OrderByMarketAndIDRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\x19\n\x08order_id\x18\x02 \x01(\tR\x07orderId"?\n\x1aOrderByMarketAndIDResponse\x12!\n\x05order\x18\x01 \x01(\x0b\x32\x0b.vega.OrderR\x05order"7\n\x17OrderByReferenceRequest\x12\x1c\n\treference\x18\x01 \x01(\tR\treference"=\n\x18OrderByReferenceResponse\x12!\n\x05order\x18\x01 \x01(\x0b\x32\x0b.vega.OrderR\x05order"\x10\n\x0eMarketsRequest"9\n\x0fMarketsResponse\x12&\n\x07markets\x18\x01 \x03(\x0b\x32\x0c.vega.MarketR\x07markets"\x82\x01\n\x0e\x43\x61ndlesRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\'\n\x0fsince_timestamp\x18\x02 \x01(\x03R\x0esinceTimestamp\x12*\n\x08interval\x18\x03 \x01(\x0e\x32\x0e.vega.IntervalR\x08interval"9\n\x0f\x43\x61ndlesResponse\x12&\n\x07\x63\x61ndles\x18\x01 \x03(\x0b\x32\x0c.vega.CandleR\x07\x63\x61ndles"N\n\x12MarketDepthRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\x1b\n\tmax_depth\x18\x02 \x01(\x04R\x08maxDepth"\xd1\x01\n\x13MarketDepthResponse\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12"\n\x03\x62uy\x18\x02 \x03(\x0b\x32\x10.vega.PriceLevelR\x03\x62uy\x12$\n\x04sell\x18\x03 \x03(\x0b\x32\x10.vega.PriceLevelR\x04sell\x12*\n\nlast_trade\x18\x04 \x01(\x0b\x32\x0b.vega.TradeR\tlastTrade\x12\'\n\x0fsequence_number\x18\x05 \x01(\x04R\x0esequenceNumber"q\n\x15TradesByMarketRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12;\n\npagination\x18\x02 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"=\n\x16TradesByMarketResponse\x12#\n\x06trades\x18\x01 \x03(\x0b\x32\x0b.vega.TradeR\x06trades"Q\n\x17PositionsByPartyRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId"H\n\x18PositionsByPartyResponse\x12,\n\tpositions\x18\x01 \x03(\x0b\x32\x0e.vega.PositionR\tpositions"\x14\n\x12GetVegaTimeRequest"3\n\x13GetVegaTimeResponse\x12\x1c\n\ttimestamp\x18\x01 \x01(\x03R\ttimestamp"V\n\nPagination\x12\x12\n\x04skip\x18\x01 \x01(\x04R\x04skip\x12\x14\n\x05limit\x18\x02 \x01(\x04R\x05limit\x12\x1e\n\ndescending\x18\x03 \x01(\x08R\ndescending">\n\x17OrdersSubscribeResponse\x12#\n\x06orders\x18\x01 \x03(\x0b\x32\x0b.vega.OrderR\x06orders">\n\x17TradesSubscribeResponse\x12#\n\x06trades\x18\x01 \x03(\x0b\x32\x0b.vega.TradeR\x06trades"!\n\x1fLedgerMovementsSubscribeRequest"a\n LedgerMovementsSubscribeResponse\x12=\n\x0fledger_movement\x18\x01 \x01(\x0b\x32\x14.vega.LedgerMovementR\x0eledgerMovement"\x8b\x01\n\x14PartyAccountsRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId\x12%\n\x04type\x18\x03 \x01(\x0e\x32\x11.vega.AccountTypeR\x04type\x12\x14\n\x05\x61sset\x18\x04 \x01(\tR\x05\x61sset"B\n\x15PartyAccountsResponse\x12)\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32\r.vega.AccountR\x08\x61\x63\x63ounts"J\n\x15MarketAccountsRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId\x12\x14\n\x05\x61sset\x18\x02 \x01(\tR\x05\x61sset"C\n\x16MarketAccountsResponse\x12)\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32\r.vega.AccountR\x08\x61\x63\x63ounts"8\n FeeInfrastructureAccountsRequest\x12\x14\n\x05\x61sset\x18\x01 \x01(\tR\x05\x61sset"N\n!FeeInfrastructureAccountsResponse\x12)\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32\r.vega.AccountR\x08\x61\x63\x63ounts"7\n\x1fGlobalRewardPoolAccountsRequest\x12\x14\n\x05\x61sset\x18\x01 \x01(\tR\x05\x61sset"M\n GlobalRewardPoolAccountsResponse\x12)\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32\r.vega.AccountR\x08\x61\x63\x63ounts"G\n\x10OrderByIDRequest\x12\x19\n\x08order_id\x18\x01 \x01(\tR\x07orderId\x12\x18\n\x07version\x18\x02 \x01(\x04R\x07version"6\n\x11OrderByIDResponse\x12!\n\x05order\x18\x01 \x01(\x0b\x32\x0b.vega.OrderR\x05order"r\n\x18OrderVersionsByIDRequest\x12\x19\n\x08order_id\x18\x01 \x01(\tR\x07orderId\x12;\n\npagination\x18\x02 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"@\n\x19OrderVersionsByIDResponse\x12#\n\x06orders\x18\x01 \x03(\x0b\x32\x0b.vega.OrderR\x06orders"7\n\x12\x45stimateFeeRequest\x12!\n\x05order\x18\x01 \x01(\x0b\x32\x0b.vega.OrderR\x05order"2\n\x13\x45stimateFeeResponse\x12\x1b\n\x03\x66\x65\x65\x18\x02 \x01(\x0b\x32\t.vega.FeeR\x03\x66\x65\x65":\n\x15\x45stimateMarginRequest\x12!\n\x05order\x18\x01 \x01(\x0b\x32\x0b.vega.OrderR\x05order"Q\n\x16\x45stimateMarginResponse\x12\x37\n\rmargin_levels\x18\x02 \x01(\x0b\x32\x12.vega.MarginLevelsR\x0cmarginLevels"\xa1\x01\n\x16ObserveEventBusRequest\x12\x30\n\x04type\x18\x01 \x03(\x0e\x32\x1c.vega.events.v1.BusEventTypeR\x04type\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId\x12\x19\n\x08party_id\x18\x03 \x01(\tR\x07partyId\x12\x1d\n\nbatch_size\x18\x04 \x01(\x03R\tbatchSize"K\n\x17ObserveEventBusResponse\x12\x30\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x18.vega.events.v1.BusEventR\x06\x65vents"/\n\x12WithdrawalsRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId"I\n\x13WithdrawalsResponse\x12\x32\n\x0bwithdrawals\x18\x01 \x03(\x0b\x32\x10.vega.WithdrawalR\x0bwithdrawals"#\n\x11WithdrawalRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"F\n\x12WithdrawalResponse\x12\x30\n\nwithdrawal\x18\x01 \x01(\x0b\x32\x10.vega.WithdrawalR\nwithdrawal"E\n\x1e\x45RC20WithdrawalApprovalRequest\x12#\n\rwithdrawal_id\x18\x01 \x01(\tR\x0cwithdrawalId"\xed\x01\n\x1f\x45RC20WithdrawalApprovalResponse\x12!\n\x0c\x61sset_source\x18\x01 \x01(\tR\x0b\x61ssetSource\x12\x16\n\x06\x61mount\x18\x02 \x01(\tR\x06\x61mount\x12\x16\n\x06\x65xpiry\x18\x03 \x01(\x03R\x06\x65xpiry\x12\x14\n\x05nonce\x18\x04 \x01(\tR\x05nonce\x12\x1e\n\nsignatures\x18\x05 \x01(\tR\nsignatures\x12%\n\x0etarget_address\x18\x06 \x01(\tR\rtargetAddress\x12\x1a\n\x08\x63reation\x18\x07 \x01(\x03R\x08\x63reation",\n\x0f\x44\x65positsRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId"=\n\x10\x44\x65positsResponse\x12)\n\x08\x64\x65posits\x18\x01 \x03(\x0b\x32\r.vega.DepositR\x08\x64\x65posits" \n\x0e\x44\x65positRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id":\n\x0f\x44\x65positResponse\x12\'\n\x07\x64\x65posit\x18\x01 \x01(\x0b\x32\r.vega.DepositR\x07\x64\x65posit"\x1a\n\x18NetworkParametersRequest"b\n\x19NetworkParametersResponse\x12\x45\n\x12network_parameters\x18\x01 \x03(\x0b\x32\x16.vega.NetworkParameterR\x11networkParameters"J\n\x1aLiquidityProvisionsRequest\x12\x16\n\x06market\x18\x01 \x01(\tR\x06market\x12\x14\n\x05party\x18\x02 \x01(\tR\x05party"j\n\x1bLiquidityProvisionsResponse\x12K\n\x14liquidity_provisions\x18\x01 \x03(\x0b\x32\x18.vega.LiquidityProvisionR\x13liquidityProvisions"`\n\x11OracleSpecRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12;\n\npagination\x18\x02 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"G\n\x12OracleSpecResponse\x12\x31\n\x0boracle_spec\x18\x01 \x01(\x0b\x32\x10.vega.OracleSpecR\noracleSpec"Q\n\x12OracleSpecsRequest\x12;\n\npagination\x18\x01 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"J\n\x13OracleSpecsResponse\x12\x33\n\x0coracle_specs\x18\x01 \x03(\x0b\x32\x10.vega.OracleSpecR\x0boracleSpecs"f\n\x17OracleDataBySpecRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12;\n\npagination\x18\x02 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"M\n\x18OracleDataBySpecResponse\x12\x31\n\x0boracle_data\x18\x01 \x03(\x0b\x32\x10.vega.OracleDataR\noracleData"T\n\x15ListOracleDataRequest\x12;\n\npagination\x18\x01 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"K\n\x16ListOracleDataResponse\x12\x31\n\x0boracle_data\x18\x01 \x03(\x0b\x32\x10.vega.OracleDataR\noracleData"H\n\x15ObserveRewardsRequest\x12\x19\n\x08\x61sset_id\x18\x01 \x01(\tR\x07\x61ssetId\x12\x14\n\x05party\x18\x02 \x01(\tR\x05party">\n\x16ObserveRewardsResponse\x12$\n\x06reward\x18\x01 \x01(\x0b\x32\x0c.vega.RewardR\x06reward"Q\n\x19GetRewardSummariesRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x19\n\x08\x61sset_id\x18\x02 \x01(\tR\x07\x61ssetId"O\n\x1aGetRewardSummariesResponse\x12\x31\n\tsummaries\x18\x01 \x03(\x0b\x32\x13.vega.RewardSummaryR\tsummaries"\x86\x01\n\x11GetRewardsRequest\x12\x19\n\x08party_id\x18\x01 \x01(\tR\x07partyId\x12\x19\n\x08\x61sset_id\x18\x02 \x01(\tR\x07\x61ssetId\x12;\n\npagination\x18\x03 \x01(\x0b\x32\x1b.datanode.api.v1.PaginationR\npagination"<\n\x12GetRewardsResponse\x12&\n\x07rewards\x18\x01 \x03(\x0b\x32\x0c.vega.RewardR\x07rewards"Z\n\nCheckpoint\x12\x12\n\x04hash\x18\x01 \x01(\tR\x04hash\x12\x1d\n\nblock_hash\x18\x02 \x01(\tR\tblockHash\x12\x19\n\x08\x61t_block\x18\x03 \x01(\x04R\x07\x61tBlock"\x14\n\x12\x43heckpointsRequest"T\n\x13\x43heckpointsResponse\x12=\n\x0b\x63heckpoints\x18\x01 \x03(\x0b\x32\x1b.datanode.api.v1.CheckpointR\x0b\x63heckpoints"4\n\x15GetRiskFactorsRequest\x12\x1b\n\tmarket_id\x18\x01 \x01(\tR\x08marketId"K\n\x16GetRiskFactorsResponse\x12\x31\n\x0brisk_factor\x18\x01 \x01(\x0b\x32\x10.vega.RiskFactorR\nriskFactor2\xf7@\n\x12TradingDataService\x12\x61\n\x0eMarketAccounts\x12&.datanode.api.v1.MarketAccountsRequest\x1a\'.datanode.api.v1.MarketAccountsResponse\x12^\n\rPartyAccounts\x12%.datanode.api.v1.PartyAccountsRequest\x1a&.datanode.api.v1.PartyAccountsResponse\x12\x82\x01\n\x19\x46\x65\x65InfrastructureAccounts\x12\x31.datanode.api.v1.FeeInfrastructureAccountsRequest\x1a\x32.datanode.api.v1.FeeInfrastructureAccountsResponse\x12\x7f\n\x18GlobalRewardPoolAccounts\x12\x30.datanode.api.v1.GlobalRewardPoolAccountsRequest\x1a\x31.datanode.api.v1.GlobalRewardPoolAccountsResponse\x12L\n\x07\x43\x61ndles\x12\x1f.datanode.api.v1.CandlesRequest\x1a .datanode.api.v1.CandlesResponse\x12\x61\n\x0eMarketDataByID\x12&.datanode.api.v1.MarketDataByIDRequest\x1a\'.datanode.api.v1.MarketDataByIDResponse\x12X\n\x0bMarketsData\x12#.datanode.api.v1.MarketsDataRequest\x1a$.datanode.api.v1.MarketsDataResponse\x12U\n\nMarketByID\x12".datanode.api.v1.MarketByIDRequest\x1a#.datanode.api.v1.MarketByIDResponse\x12X\n\x0bMarketDepth\x12#.datanode.api.v1.MarketDepthRequest\x1a$.datanode.api.v1.MarketDepthResponse\x12L\n\x07Markets\x12\x1f.datanode.api.v1.MarketsRequest\x1a .datanode.api.v1.MarketsResponse\x12m\n\x12OrderByMarketAndID\x12*.datanode.api.v1.OrderByMarketAndIDRequest\x1a+.datanode.api.v1.OrderByMarketAndIDResponse\x12g\n\x10OrderByReference\x12(.datanode.api.v1.OrderByReferenceRequest\x1a).datanode.api.v1.OrderByReferenceResponse\x12\x61\n\x0eOrdersByMarket\x12&.datanode.api.v1.OrdersByMarketRequest\x1a\'.datanode.api.v1.OrdersByMarketResponse\x12^\n\rOrdersByParty\x12%.datanode.api.v1.OrdersByPartyRequest\x1a&.datanode.api.v1.OrdersByPartyResponse\x12R\n\tOrderByID\x12!.datanode.api.v1.OrderByIDRequest\x1a".datanode.api.v1.OrderByIDResponse\x12j\n\x11OrderVersionsByID\x12).datanode.api.v1.OrderVersionsByIDRequest\x1a*.datanode.api.v1.OrderVersionsByIDResponse\x12[\n\x0cMarginLevels\x12$.datanode.api.v1.MarginLevelsRequest\x1a%.datanode.api.v1.MarginLevelsResponse\x12L\n\x07Parties\x12\x1f.datanode.api.v1.PartiesRequest\x1a .datanode.api.v1.PartiesResponse\x12R\n\tPartyByID\x12!.datanode.api.v1.PartyByIDRequest\x1a".datanode.api.v1.PartyByIDResponse\x12g\n\x10PositionsByParty\x12(.datanode.api.v1.PositionsByPartyRequest\x1a).datanode.api.v1.PositionsByPartyResponse\x12R\n\tLastTrade\x12!.datanode.api.v1.LastTradeRequest\x1a".datanode.api.v1.LastTradeResponse\x12\x61\n\x0eTradesByMarket\x12&.datanode.api.v1.TradesByMarketRequest\x1a\'.datanode.api.v1.TradesByMarketResponse\x12^\n\rTradesByOrder\x12%.datanode.api.v1.TradesByOrderRequest\x1a&.datanode.api.v1.TradesByOrderResponse\x12^\n\rTradesByParty\x12%.datanode.api.v1.TradesByPartyRequest\x1a&.datanode.api.v1.TradesByPartyResponse\x12[\n\x0cGetProposals\x12$.datanode.api.v1.GetProposalsRequest\x1a%.datanode.api.v1.GetProposalsResponse\x12p\n\x13GetProposalsByParty\x12+.datanode.api.v1.GetProposalsByPartyRequest\x1a,.datanode.api.v1.GetProposalsByPartyResponse\x12\x64\n\x0fGetVotesByParty\x12\'.datanode.api.v1.GetVotesByPartyRequest\x1a(.datanode.api.v1.GetVotesByPartyResponse\x12v\n\x15GetNewMarketProposals\x12-.datanode.api.v1.GetNewMarketProposalsRequest\x1a..datanode.api.v1.GetNewMarketProposalsResponse\x12\x7f\n\x18GetUpdateMarketProposals\x12\x30.datanode.api.v1.GetUpdateMarketProposalsRequest\x1a\x31.datanode.api.v1.GetUpdateMarketProposalsResponse\x12\x8e\x01\n\x1dGetNetworkParametersProposals\x12\x35.datanode.api.v1.GetNetworkParametersProposalsRequest\x1a\x36.datanode.api.v1.GetNetworkParametersProposalsResponse\x12s\n\x14GetNewAssetProposals\x12,.datanode.api.v1.GetNewAssetProposalsRequest\x1a-.datanode.api.v1.GetNewAssetProposalsResponse\x12|\n\x17GetNewFreeformProposals\x12/.datanode.api.v1.GetNewFreeformProposalsRequest\x1a\x30.datanode.api.v1.GetNewFreeformProposalsResponse\x12\x64\n\x0fGetProposalByID\x12\'.datanode.api.v1.GetProposalByIDRequest\x1a(.datanode.api.v1.GetProposalByIDResponse\x12y\n\x16GetProposalByReference\x12..datanode.api.v1.GetProposalByReferenceRequest\x1a/.datanode.api.v1.GetProposalByReferenceResponse\x12l\n\x11ObserveGovernance\x12).datanode.api.v1.ObserveGovernanceRequest\x1a*.datanode.api.v1.ObserveGovernanceResponse0\x01\x12x\n\x15ObservePartyProposals\x12-.datanode.api.v1.ObservePartyProposalsRequest\x1a..datanode.api.v1.ObservePartyProposalsResponse0\x01\x12l\n\x11ObservePartyVotes\x12).datanode.api.v1.ObservePartyVotesRequest\x1a*.datanode.api.v1.ObservePartyVotesResponse0\x01\x12u\n\x14ObserveProposalVotes\x12,.datanode.api.v1.ObserveProposalVotesRequest\x1a-.datanode.api.v1.ObserveProposalVotesResponse0\x01\x12h\n\x0fObserveEventBus\x12\'.datanode.api.v1.ObserveEventBusRequest\x1a(.datanode.api.v1.ObserveEventBusResponse(\x01\x30\x01\x12X\n\x0bGetNodeData\x12#.datanode.api.v1.GetNodeDataRequest\x1a$.datanode.api.v1.GetNodeDataResponse\x12O\n\x08GetNodes\x12 .datanode.api.v1.GetNodesRequest\x1a!.datanode.api.v1.GetNodesResponse\x12X\n\x0bGetNodeByID\x12#.datanode.api.v1.GetNodeByIDRequest\x1a$.datanode.api.v1.GetNodeByIDResponse\x12\x64\n\x0fGetKeyRotations\x12\'.datanode.api.v1.GetKeyRotationsRequest\x1a(.datanode.api.v1.GetKeyRotationsResponse\x12v\n\x15GetKeyRotationsByNode\x12-.datanode.api.v1.GetKeyRotationsByNodeRequest\x1a..datanode.api.v1.GetKeyRotationsByNodeResponse\x12O\n\x08GetEpoch\x12 .datanode.api.v1.GetEpochRequest\x1a!.datanode.api.v1.GetEpochResponse\x12X\n\x0bGetVegaTime\x12#.datanode.api.v1.GetVegaTimeRequest\x1a$.datanode.api.v1.GetVegaTimeResponse\x12l\n\x11\x41\x63\x63ountsSubscribe\x12).datanode.api.v1.AccountsSubscribeRequest\x1a*.datanode.api.v1.AccountsSubscribeResponse0\x01\x12i\n\x10\x43\x61ndlesSubscribe\x12(.datanode.api.v1.CandlesSubscribeRequest\x1a).datanode.api.v1.CandlesSubscribeResponse0\x01\x12x\n\x15MarginLevelsSubscribe\x12-.datanode.api.v1.MarginLevelsSubscribeRequest\x1a..datanode.api.v1.MarginLevelsSubscribeResponse0\x01\x12u\n\x14MarketDepthSubscribe\x12,.datanode.api.v1.MarketDepthSubscribeRequest\x1a-.datanode.api.v1.MarketDepthSubscribeResponse0\x01\x12\x8a\x01\n\x1bMarketDepthUpdatesSubscribe\x12\x33.datanode.api.v1.MarketDepthUpdatesSubscribeRequest\x1a\x34.datanode.api.v1.MarketDepthUpdatesSubscribeResponse0\x01\x12u\n\x14MarketsDataSubscribe\x12,.datanode.api.v1.MarketsDataSubscribeRequest\x1a-.datanode.api.v1.MarketsDataSubscribeResponse0\x01\x12\x66\n\x0fOrdersSubscribe\x12\'.datanode.api.v1.OrdersSubscribeRequest\x1a(.datanode.api.v1.OrdersSubscribeResponse0\x01\x12o\n\x12PositionsSubscribe\x12*.datanode.api.v1.PositionsSubscribeRequest\x1a+.datanode.api.v1.PositionsSubscribeResponse0\x01\x12\x66\n\x0fTradesSubscribe\x12\'.datanode.api.v1.TradesSubscribeRequest\x1a(.datanode.api.v1.TradesSubscribeResponse0\x01\x12\x81\x01\n\x18LedgerMovementsSubscribe\x12\x30.datanode.api.v1.LedgerMovementsSubscribeRequest\x1a\x31.datanode.api.v1.LedgerMovementsSubscribeResponse0\x01\x12\x85\x01\n\x1aGetNodeSignaturesAggregate\x12\x32.datanode.api.v1.GetNodeSignaturesAggregateRequest\x1a\x33.datanode.api.v1.GetNodeSignaturesAggregateResponse\x12R\n\tAssetByID\x12!.datanode.api.v1.AssetByIDRequest\x1a".datanode.api.v1.AssetByIDResponse\x12I\n\x06\x41ssets\x12\x1e.datanode.api.v1.AssetsRequest\x1a\x1f.datanode.api.v1.AssetsResponse\x12X\n\x0b\x45stimateFee\x12#.datanode.api.v1.EstimateFeeRequest\x1a$.datanode.api.v1.EstimateFeeResponse\x12\x61\n\x0e\x45stimateMargin\x12&.datanode.api.v1.EstimateMarginRequest\x1a\'.datanode.api.v1.EstimateMarginResponse\x12|\n\x17\x45RC20WithdrawalApproval\x12/.datanode.api.v1.ERC20WithdrawalApprovalRequest\x1a\x30.datanode.api.v1.ERC20WithdrawalApprovalResponse\x12U\n\nWithdrawal\x12".datanode.api.v1.WithdrawalRequest\x1a#.datanode.api.v1.WithdrawalResponse\x12X\n\x0bWithdrawals\x12#.datanode.api.v1.WithdrawalsRequest\x1a$.datanode.api.v1.WithdrawalsResponse\x12L\n\x07\x44\x65posit\x12\x1f.datanode.api.v1.DepositRequest\x1a .datanode.api.v1.DepositResponse\x12O\n\x08\x44\x65posits\x12 .datanode.api.v1.DepositsRequest\x1a!.datanode.api.v1.DepositsResponse\x12j\n\x11NetworkParameters\x12).datanode.api.v1.NetworkParametersRequest\x1a*.datanode.api.v1.NetworkParametersResponse\x12p\n\x13LiquidityProvisions\x12+.datanode.api.v1.LiquidityProvisionsRequest\x1a,.datanode.api.v1.LiquidityProvisionsResponse\x12U\n\nOracleSpec\x12".datanode.api.v1.OracleSpecRequest\x1a#.datanode.api.v1.OracleSpecResponse\x12X\n\x0bOracleSpecs\x12#.datanode.api.v1.OracleSpecsRequest\x1a$.datanode.api.v1.OracleSpecsResponse\x12g\n\x10OracleDataBySpec\x12(.datanode.api.v1.OracleDataBySpecRequest\x1a).datanode.api.v1.OracleDataBySpecResponse\x12\x61\n\x0eListOracleData\x12&.datanode.api.v1.ListOracleDataRequest\x1a\'.datanode.api.v1.ListOracleDataResponse\x12\x63\n\x0eObserveRewards\x12&.datanode.api.v1.ObserveRewardsRequest\x1a\'.datanode.api.v1.ObserveRewardsResponse0\x01\x12U\n\nGetRewards\x12".datanode.api.v1.GetRewardsRequest\x1a#.datanode.api.v1.GetRewardsResponse\x12m\n\x12GetRewardSummaries\x12*.datanode.api.v1.GetRewardSummariesRequest\x1a+.datanode.api.v1.GetRewardSummariesResponse\x12X\n\x0b\x43heckpoints\x12#.datanode.api.v1.CheckpointsRequest\x1a$.datanode.api.v1.CheckpointsResponse\x12X\n\x0b\x44\x65legations\x12#.datanode.api.v1.DelegationsRequest\x1a$.datanode.api.v1.DelegationsResponse\x12o\n\x12ObserveDelegations\x12*.datanode.api.v1.ObserveDelegationsRequest\x1a+.datanode.api.v1.ObserveDelegationsResponse0\x01\x12U\n\nPartyStake\x12".datanode.api.v1.PartyStakeRequest\x1a#.datanode.api.v1.PartyStakeResponse\x12R\n\tTransfers\x12!.datanode.api.v1.TransfersRequest\x1a".datanode.api.v1.TransfersResponse\x12\x61\n\x0eGetRiskFactors\x12&.datanode.api.v1.GetRiskFactorsRequest\x1a\'.datanode.api.v1.GetRiskFactorsResponseB3Z1code.vegaprotocol.io/vega/protos/data-node/api/v1b\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "data_node.api.v1.trading_data_pb2", globals()
)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = (
        b"Z1code.vegaprotocol.io/vega/protos/data-node/api/v1"
    )
    _TRANSFERSREQUEST._serialized_start = 226
    _TRANSFERSREQUEST._serialized_end = 314
    _TRANSFERSRESPONSE._serialized_start = 316
    _TRANSFERSRESPONSE._serialized_end = 391
    _DELEGATIONSREQUEST._serialized_start = 394
    _DELEGATIONSREQUEST._serialized_end = 551
    _DELEGATIONSRESPONSE._serialized_start = 553
    _DELEGATIONSRESPONSE._serialized_end = 626
    _OBSERVEDELEGATIONSREQUEST._serialized_start = 628
    _OBSERVEDELEGATIONSREQUEST._serialized_end = 702
    _OBSERVEDELEGATIONSRESPONSE._serialized_start = 704
    _OBSERVEDELEGATIONSRESPONSE._serialized_end = 782
    _PARTYSTAKEREQUEST._serialized_start = 784
    _PARTYSTAKEREQUEST._serialized_end = 825
    _PARTYSTAKERESPONSE._serialized_start = 828
    _PARTYSTAKERESPONSE._serialized_end = 973
    _GETNODEDATAREQUEST._serialized_start = 975
    _GETNODEDATAREQUEST._serialized_end = 995
    _GETNODEDATARESPONSE._serialized_start = 997
    _GETNODEDATARESPONSE._serialized_end = 1063
    _GETNODESREQUEST._serialized_start = 1065
    _GETNODESREQUEST._serialized_end = 1082
    _GETNODESRESPONSE._serialized_start = 1084
    _GETNODESRESPONSE._serialized_end = 1136
    _GETNODEBYIDREQUEST._serialized_start = 1138
    _GETNODEBYIDREQUEST._serialized_end = 1174
    _GETNODEBYIDRESPONSE._serialized_start = 1176
    _GETNODEBYIDRESPONSE._serialized_end = 1229
    _GETEPOCHREQUEST._serialized_start = 1231
    _GETEPOCHREQUEST._serialized_end = 1264
    _GETEPOCHRESPONSE._serialized_start = 1266
    _GETEPOCHRESPONSE._serialized_end = 1319
    _KEYROTATION._serialized_start = 1322
    _KEYROTATION._serialized_end = 1459
    _GETKEYROTATIONSREQUEST._serialized_start = 1461
    _GETKEYROTATIONSREQUEST._serialized_end = 1485
    _GETKEYROTATIONSRESPONSE._serialized_start = 1487
    _GETKEYROTATIONSRESPONSE._serialized_end = 1572
    _GETKEYROTATIONSBYNODEREQUEST._serialized_start = 1574
    _GETKEYROTATIONSBYNODEREQUEST._serialized_end = 1629
    _GETKEYROTATIONSBYNODERESPONSE._serialized_start = 1631
    _GETKEYROTATIONSBYNODERESPONSE._serialized_end = 1722
    _ASSETSREQUEST._serialized_start = 1724
    _ASSETSREQUEST._serialized_end = 1739
    _ASSETSRESPONSE._serialized_start = 1741
    _ASSETSRESPONSE._serialized_end = 1794
    _ASSETBYIDREQUEST._serialized_start = 1796
    _ASSETBYIDREQUEST._serialized_end = 1830
    _ASSETBYIDRESPONSE._serialized_start = 1832
    _ASSETBYIDRESPONSE._serialized_end = 1886
    _GETNODESIGNATURESAGGREGATEREQUEST._serialized_start = 1888
    _GETNODESIGNATURESAGGREGATEREQUEST._serialized_end = 1939
    _GETNODESIGNATURESAGGREGATERESPONSE._serialized_start = 1941
    _GETNODESIGNATURESAGGREGATERESPONSE._serialized_end = 2042
    _GETPROPOSALSREQUEST._serialized_start = 2044
    _GETPROPOSALSREQUEST._serialized_end = 2152
    _GETPROPOSALSRESPONSE._serialized_start = 2154
    _GETPROPOSALSRESPONSE._serialized_end = 2218
    _GETPROPOSALSBYPARTYREQUEST._serialized_start = 2221
    _GETPROPOSALSBYPARTYREQUEST._serialized_end = 2363
    _GETPROPOSALSBYPARTYRESPONSE._serialized_start = 2365
    _GETPROPOSALSBYPARTYRESPONSE._serialized_end = 2436
    _GETVOTESBYPARTYREQUEST._serialized_start = 2438
    _GETVOTESBYPARTYREQUEST._serialized_end = 2489
    _GETVOTESBYPARTYRESPONSE._serialized_start = 2491
    _GETVOTESBYPARTYRESPONSE._serialized_end = 2550
    _GETNEWMARKETPROPOSALSREQUEST._serialized_start = 2552
    _GETNEWMARKETPROPOSALSREQUEST._serialized_end = 2669
    _GETNEWMARKETPROPOSALSRESPONSE._serialized_start = 2671
    _GETNEWMARKETPROPOSALSRESPONSE._serialized_end = 2744
    _GETUPDATEMARKETPROPOSALSREQUEST._serialized_start = 2747
    _GETUPDATEMARKETPROPOSALSREQUEST._serialized_end = 2896
    _GETUPDATEMARKETPROPOSALSRESPONSE._serialized_start = 2898
    _GETUPDATEMARKETPROPOSALSRESPONSE._serialized_end = 2974
    _GETNETWORKPARAMETERSPROPOSALSREQUEST._serialized_start = 2976
    _GETNETWORKPARAMETERSPROPOSALSREQUEST._serialized_end = 3101
    _GETNETWORKPARAMETERSPROPOSALSRESPONSE._serialized_start = 3103
    _GETNETWORKPARAMETERSPROPOSALSRESPONSE._serialized_end = 3184
    _GETNEWFREEFORMPROPOSALSREQUEST._serialized_start = 3186
    _GETNEWFREEFORMPROPOSALSREQUEST._serialized_end = 3305
    _GETNEWFREEFORMPROPOSALSRESPONSE._serialized_start = 3307
    _GETNEWFREEFORMPROPOSALSRESPONSE._serialized_end = 3382
    _GETNEWASSETPROPOSALSREQUEST._serialized_start = 3384
    _GETNEWASSETPROPOSALSREQUEST._serialized_end = 3500
    _GETNEWASSETPROPOSALSRESPONSE._serialized_start = 3502
    _GETNEWASSETPROPOSALSRESPONSE._serialized_end = 3574
    _GETPROPOSALBYIDREQUEST._serialized_start = 3576
    _GETPROPOSALBYIDREQUEST._serialized_end = 3633
    _GETPROPOSALBYIDRESPONSE._serialized_start = 3635
    _GETPROPOSALBYIDRESPONSE._serialized_end = 3702
    _GETPROPOSALBYREFERENCEREQUEST._serialized_start = 3704
    _GETPROPOSALBYREFERENCEREQUEST._serialized_end = 3765
    _GETPROPOSALBYREFERENCERESPONSE._serialized_start = 3767
    _GETPROPOSALBYREFERENCERESPONSE._serialized_end = 3841
    _OBSERVEGOVERNANCEREQUEST._serialized_start = 3843
    _OBSERVEGOVERNANCEREQUEST._serialized_end = 3869
    _OBSERVEGOVERNANCERESPONSE._serialized_start = 3871
    _OBSERVEGOVERNANCERESPONSE._serialized_end = 3940
    _OBSERVEPARTYPROPOSALSREQUEST._serialized_start = 3942
    _OBSERVEPARTYPROPOSALSREQUEST._serialized_end = 3999
    _OBSERVEPARTYPROPOSALSRESPONSE._serialized_start = 4001
    _OBSERVEPARTYPROPOSALSRESPONSE._serialized_end = 4074
    _OBSERVEPROPOSALVOTESREQUEST._serialized_start = 4076
    _OBSERVEPROPOSALVOTESREQUEST._serialized_end = 4138
    _OBSERVEPROPOSALVOTESRESPONSE._serialized_start = 4140
    _OBSERVEPROPOSALVOTESRESPONSE._serialized_end = 4202
    _OBSERVEPARTYVOTESREQUEST._serialized_start = 4204
    _OBSERVEPARTYVOTESREQUEST._serialized_end = 4257
    _OBSERVEPARTYVOTESRESPONSE._serialized_start = 4259
    _OBSERVEPARTYVOTESRESPONSE._serialized_end = 4318
    _MARGINLEVELSSUBSCRIBEREQUEST._serialized_start = 4320
    _MARGINLEVELSSUBSCRIBEREQUEST._serialized_end = 4406
    _MARGINLEVELSSUBSCRIBERESPONSE._serialized_start = 4408
    _MARGINLEVELSSUBSCRIBERESPONSE._serialized_end = 4496
    _MARGINLEVELSREQUEST._serialized_start = 4498
    _MARGINLEVELSREQUEST._serialized_end = 4575
    _MARGINLEVELSRESPONSE._serialized_start = 4577
    _MARGINLEVELSRESPONSE._serialized_end = 4656
    _MARKETSDATASUBSCRIBEREQUEST._serialized_start = 4658
    _MARKETSDATASUBSCRIBEREQUEST._serialized_end = 4716
    _MARKETSDATASUBSCRIBERESPONSE._serialized_start = 4718
    _MARKETSDATASUBSCRIBERESPONSE._serialized_end = 4799
    _MARKETDATABYIDREQUEST._serialized_start = 4801
    _MARKETDATABYIDREQUEST._serialized_end = 4853
    _MARKETDATABYIDRESPONSE._serialized_start = 4855
    _MARKETDATABYIDRESPONSE._serialized_end = 4930
    _MARKETSDATAREQUEST._serialized_start = 4932
    _MARKETSDATAREQUEST._serialized_end = 4952
    _MARKETSDATARESPONSE._serialized_start = 4954
    _MARKETSDATARESPONSE._serialized_end = 5028
    _LASTTRADEREQUEST._serialized_start = 5030
    _LASTTRADEREQUEST._serialized_end = 5077
    _LASTTRADERESPONSE._serialized_start = 5079
    _LASTTRADERESPONSE._serialized_end = 5133
    _MARKETBYIDREQUEST._serialized_start = 5135
    _MARKETBYIDREQUEST._serialized_end = 5183
    _MARKETBYIDRESPONSE._serialized_start = 5185
    _MARKETBYIDRESPONSE._serialized_end = 5243
    _PARTYBYIDREQUEST._serialized_start = 5245
    _PARTYBYIDREQUEST._serialized_end = 5290
    _PARTYBYIDRESPONSE._serialized_start = 5292
    _PARTYBYIDRESPONSE._serialized_end = 5346
    _PARTIESREQUEST._serialized_start = 5348
    _PARTIESREQUEST._serialized_end = 5364
    _PARTIESRESPONSE._serialized_start = 5366
    _PARTIESRESPONSE._serialized_end = 5422
    _TRADESBYPARTYREQUEST._serialized_start = 5425
    _TRADESBYPARTYREQUEST._serialized_end = 5564
    _TRADESBYPARTYRESPONSE._serialized_start = 5566
    _TRADESBYPARTYRESPONSE._serialized_end = 5626
    _TRADESBYORDERREQUEST._serialized_start = 5628
    _TRADESBYORDERREQUEST._serialized_end = 5677
    _TRADESBYORDERRESPONSE._serialized_start = 5679
    _TRADESBYORDERRESPONSE._serialized_end = 5739
    _ACCOUNTSSUBSCRIBEREQUEST._serialized_start = 5742
    _ACCOUNTSSUBSCRIBEREQUEST._serialized_end = 5885
    _ACCOUNTSSUBSCRIBERESPONSE._serialized_start = 5887
    _ACCOUNTSSUBSCRIBERESPONSE._serialized_end = 5955
    _ORDERSSUBSCRIBEREQUEST._serialized_start = 5957
    _ORDERSSUBSCRIBEREQUEST._serialized_end = 6037
    _TRADESSUBSCRIBEREQUEST._serialized_start = 6039
    _TRADESSUBSCRIBEREQUEST._serialized_end = 6119
    _CANDLESSUBSCRIBEREQUEST._serialized_start = 6121
    _CANDLESSUBSCRIBEREQUEST._serialized_end = 6219
    _CANDLESSUBSCRIBERESPONSE._serialized_start = 6221
    _CANDLESSUBSCRIBERESPONSE._serialized_end = 6285
    _MARKETDEPTHSUBSCRIBEREQUEST._serialized_start = 6287
    _MARKETDEPTHSUBSCRIBEREQUEST._serialized_end = 6345
    _MARKETDEPTHSUBSCRIBERESPONSE._serialized_start = 6347
    _MARKETDEPTHSUBSCRIBERESPONSE._serialized_end = 6431
    _MARKETDEPTHUPDATESSUBSCRIBEREQUEST._serialized_start = 6433
    _MARKETDEPTHUPDATESSUBSCRIBEREQUEST._serialized_end = 6498
    _MARKETDEPTHUPDATESSUBSCRIBERESPONSE._serialized_start = 6500
    _MARKETDEPTHUPDATESSUBSCRIBERESPONSE._serialized_end = 6586
    _POSITIONSSUBSCRIBEREQUEST._serialized_start = 6588
    _POSITIONSSUBSCRIBEREQUEST._serialized_end = 6671
    _POSITIONSSUBSCRIBERESPONSE._serialized_start = 6673
    _POSITIONSSUBSCRIBERESPONSE._serialized_end = 6745
    _ORDERSBYMARKETREQUEST._serialized_start = 6747
    _ORDERSBYMARKETREQUEST._serialized_end = 6860
    _ORDERSBYMARKETRESPONSE._serialized_start = 6862
    _ORDERSBYMARKETRESPONSE._serialized_end = 6923
    _ORDERSBYPARTYREQUEST._serialized_start = 6925
    _ORDERSBYPARTYREQUEST._serialized_end = 7035
    _ORDERSBYPARTYRESPONSE._serialized_start = 7037
    _ORDERSBYPARTYRESPONSE._serialized_end = 7097
    _ORDERBYMARKETANDIDREQUEST._serialized_start = 7099
    _ORDERBYMARKETANDIDREQUEST._serialized_end = 7182
    _ORDERBYMARKETANDIDRESPONSE._serialized_start = 7184
    _ORDERBYMARKETANDIDRESPONSE._serialized_end = 7247
    _ORDERBYREFERENCEREQUEST._serialized_start = 7249
    _ORDERBYREFERENCEREQUEST._serialized_end = 7304
    _ORDERBYREFERENCERESPONSE._serialized_start = 7306
    _ORDERBYREFERENCERESPONSE._serialized_end = 7367
    _MARKETSREQUEST._serialized_start = 7369
    _MARKETSREQUEST._serialized_end = 7385
    _MARKETSRESPONSE._serialized_start = 7387
    _MARKETSRESPONSE._serialized_end = 7444
    _CANDLESREQUEST._serialized_start = 7447
    _CANDLESREQUEST._serialized_end = 7577
    _CANDLESRESPONSE._serialized_start = 7579
    _CANDLESRESPONSE._serialized_end = 7636
    _MARKETDEPTHREQUEST._serialized_start = 7638
    _MARKETDEPTHREQUEST._serialized_end = 7716
    _MARKETDEPTHRESPONSE._serialized_start = 7719
    _MARKETDEPTHRESPONSE._serialized_end = 7928
    _TRADESBYMARKETREQUEST._serialized_start = 7930
    _TRADESBYMARKETREQUEST._serialized_end = 8043
    _TRADESBYMARKETRESPONSE._serialized_start = 8045
    _TRADESBYMARKETRESPONSE._serialized_end = 8106
    _POSITIONSBYPARTYREQUEST._serialized_start = 8108
    _POSITIONSBYPARTYREQUEST._serialized_end = 8189
    _POSITIONSBYPARTYRESPONSE._serialized_start = 8191
    _POSITIONSBYPARTYRESPONSE._serialized_end = 8263
    _GETVEGATIMEREQUEST._serialized_start = 8265
    _GETVEGATIMEREQUEST._serialized_end = 8285
    _GETVEGATIMERESPONSE._serialized_start = 8287
    _GETVEGATIMERESPONSE._serialized_end = 8338
    _PAGINATION._serialized_start = 8340
    _PAGINATION._serialized_end = 8426
    _ORDERSSUBSCRIBERESPONSE._serialized_start = 8428
    _ORDERSSUBSCRIBERESPONSE._serialized_end = 8490
    _TRADESSUBSCRIBERESPONSE._serialized_start = 8492
    _TRADESSUBSCRIBERESPONSE._serialized_end = 8554
    _LEDGERMOVEMENTSSUBSCRIBEREQUEST._serialized_start = 8556
    _LEDGERMOVEMENTSSUBSCRIBEREQUEST._serialized_end = 8589
    _LEDGERMOVEMENTSSUBSCRIBERESPONSE._serialized_start = 8591
    _LEDGERMOVEMENTSSUBSCRIBERESPONSE._serialized_end = 8688
    _PARTYACCOUNTSREQUEST._serialized_start = 8691
    _PARTYACCOUNTSREQUEST._serialized_end = 8830
    _PARTYACCOUNTSRESPONSE._serialized_start = 8832
    _PARTYACCOUNTSRESPONSE._serialized_end = 8898
    _MARKETACCOUNTSREQUEST._serialized_start = 8900
    _MARKETACCOUNTSREQUEST._serialized_end = 8974
    _MARKETACCOUNTSRESPONSE._serialized_start = 8976
    _MARKETACCOUNTSRESPONSE._serialized_end = 9043
    _FEEINFRASTRUCTUREACCOUNTSREQUEST._serialized_start = 9045
    _FEEINFRASTRUCTUREACCOUNTSREQUEST._serialized_end = 9101
    _FEEINFRASTRUCTUREACCOUNTSRESPONSE._serialized_start = 9103
    _FEEINFRASTRUCTUREACCOUNTSRESPONSE._serialized_end = 9181
    _GLOBALREWARDPOOLACCOUNTSREQUEST._serialized_start = 9183
    _GLOBALREWARDPOOLACCOUNTSREQUEST._serialized_end = 9238
    _GLOBALREWARDPOOLACCOUNTSRESPONSE._serialized_start = 9240
    _GLOBALREWARDPOOLACCOUNTSRESPONSE._serialized_end = 9317
    _ORDERBYIDREQUEST._serialized_start = 9319
    _ORDERBYIDREQUEST._serialized_end = 9390
    _ORDERBYIDRESPONSE._serialized_start = 9392
    _ORDERBYIDRESPONSE._serialized_end = 9446
    _ORDERVERSIONSBYIDREQUEST._serialized_start = 9448
    _ORDERVERSIONSBYIDREQUEST._serialized_end = 9562
    _ORDERVERSIONSBYIDRESPONSE._serialized_start = 9564
    _ORDERVERSIONSBYIDRESPONSE._serialized_end = 9628
    _ESTIMATEFEEREQUEST._serialized_start = 9630
    _ESTIMATEFEEREQUEST._serialized_end = 9685
    _ESTIMATEFEERESPONSE._serialized_start = 9687
    _ESTIMATEFEERESPONSE._serialized_end = 9737
    _ESTIMATEMARGINREQUEST._serialized_start = 9739
    _ESTIMATEMARGINREQUEST._serialized_end = 9797
    _ESTIMATEMARGINRESPONSE._serialized_start = 9799
    _ESTIMATEMARGINRESPONSE._serialized_end = 9880
    _OBSERVEEVENTBUSREQUEST._serialized_start = 9883
    _OBSERVEEVENTBUSREQUEST._serialized_end = 10044
    _OBSERVEEVENTBUSRESPONSE._serialized_start = 10046
    _OBSERVEEVENTBUSRESPONSE._serialized_end = 10121
    _WITHDRAWALSREQUEST._serialized_start = 10123
    _WITHDRAWALSREQUEST._serialized_end = 10170
    _WITHDRAWALSRESPONSE._serialized_start = 10172
    _WITHDRAWALSRESPONSE._serialized_end = 10245
    _WITHDRAWALREQUEST._serialized_start = 10247
    _WITHDRAWALREQUEST._serialized_end = 10282
    _WITHDRAWALRESPONSE._serialized_start = 10284
    _WITHDRAWALRESPONSE._serialized_end = 10354
    _ERC20WITHDRAWALAPPROVALREQUEST._serialized_start = 10356
    _ERC20WITHDRAWALAPPROVALREQUEST._serialized_end = 10425
    _ERC20WITHDRAWALAPPROVALRESPONSE._serialized_start = 10428
    _ERC20WITHDRAWALAPPROVALRESPONSE._serialized_end = 10665
    _DEPOSITSREQUEST._serialized_start = 10667
    _DEPOSITSREQUEST._serialized_end = 10711
    _DEPOSITSRESPONSE._serialized_start = 10713
    _DEPOSITSRESPONSE._serialized_end = 10774
    _DEPOSITREQUEST._serialized_start = 10776
    _DEPOSITREQUEST._serialized_end = 10808
    _DEPOSITRESPONSE._serialized_start = 10810
    _DEPOSITRESPONSE._serialized_end = 10868
    _NETWORKPARAMETERSREQUEST._serialized_start = 10870
    _NETWORKPARAMETERSREQUEST._serialized_end = 10896
    _NETWORKPARAMETERSRESPONSE._serialized_start = 10898
    _NETWORKPARAMETERSRESPONSE._serialized_end = 10996
    _LIQUIDITYPROVISIONSREQUEST._serialized_start = 10998
    _LIQUIDITYPROVISIONSREQUEST._serialized_end = 11072
    _LIQUIDITYPROVISIONSRESPONSE._serialized_start = 11074
    _LIQUIDITYPROVISIONSRESPONSE._serialized_end = 11180
    _ORACLESPECREQUEST._serialized_start = 11182
    _ORACLESPECREQUEST._serialized_end = 11278
    _ORACLESPECRESPONSE._serialized_start = 11280
    _ORACLESPECRESPONSE._serialized_end = 11351
    _ORACLESPECSREQUEST._serialized_start = 11353
    _ORACLESPECSREQUEST._serialized_end = 11434
    _ORACLESPECSRESPONSE._serialized_start = 11436
    _ORACLESPECSRESPONSE._serialized_end = 11510
    _ORACLEDATABYSPECREQUEST._serialized_start = 11512
    _ORACLEDATABYSPECREQUEST._serialized_end = 11614
    _ORACLEDATABYSPECRESPONSE._serialized_start = 11616
    _ORACLEDATABYSPECRESPONSE._serialized_end = 11693
    _LISTORACLEDATAREQUEST._serialized_start = 11695
    _LISTORACLEDATAREQUEST._serialized_end = 11779
    _LISTORACLEDATARESPONSE._serialized_start = 11781
    _LISTORACLEDATARESPONSE._serialized_end = 11856
    _OBSERVEREWARDSREQUEST._serialized_start = 11858
    _OBSERVEREWARDSREQUEST._serialized_end = 11930
    _OBSERVEREWARDSRESPONSE._serialized_start = 11932
    _OBSERVEREWARDSRESPONSE._serialized_end = 11994
    _GETREWARDSUMMARIESREQUEST._serialized_start = 11996
    _GETREWARDSUMMARIESREQUEST._serialized_end = 12077
    _GETREWARDSUMMARIESRESPONSE._serialized_start = 12079
    _GETREWARDSUMMARIESRESPONSE._serialized_end = 12158
    _GETREWARDSREQUEST._serialized_start = 12161
    _GETREWARDSREQUEST._serialized_end = 12295
    _GETREWARDSRESPONSE._serialized_start = 12297
    _GETREWARDSRESPONSE._serialized_end = 12357
    _CHECKPOINT._serialized_start = 12359
    _CHECKPOINT._serialized_end = 12449
    _CHECKPOINTSREQUEST._serialized_start = 12451
    _CHECKPOINTSREQUEST._serialized_end = 12471
    _CHECKPOINTSRESPONSE._serialized_start = 12473
    _CHECKPOINTSRESPONSE._serialized_end = 12557
    _GETRISKFACTORSREQUEST._serialized_start = 12559
    _GETRISKFACTORSREQUEST._serialized_end = 12611
    _GETRISKFACTORSRESPONSE._serialized_start = 12613
    _GETRISKFACTORSRESPONSE._serialized_end = 12688
    _TRADINGDATASERVICE._serialized_start = 12691
    _TRADINGDATASERVICE._serialized_end = 21002
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vega/chain_events.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x17vega/chain_events.proto\x12\x04vega"l\n\x13\x42uiltinAssetDeposit\x12"\n\rvega_asset_id\x18\x01 \x01(\tR\x0bvegaAssetId\x12\x19\n\x08party_id\x18\x02 \x01(\tR\x07partyId\x12\x16\n\x06\x61mount\x18\x03 \x01(\tR\x06\x61mount"o\n\x16\x42uiltinAssetWithdrawal\x12"\n\rvega_asset_id\x18\x01 \x01(\tR\x0bvegaAssetId\x12\x19\n\x08party_id\x18\x02 \x01(\tR\x07partyId\x12\x16\n\x06\x61mount\x18\x03 \x01(\tR\x06\x61mount"\x96\x01\n\x11\x42uiltinAssetEvent\x12\x36\n\x07\x64\x65posit\x18\xe9\x07 \x01(\x0b\x32\x19.vega.BuiltinAssetDepositH\x00R\x07\x64\x65posit\x12?\n\nwithdrawal\x18\xea\x07 \x01(\x0b\x32\x1c.vega.BuiltinAssetWithdrawalH\x00R\nwithdrawalB\x08\n\x06\x61\x63tion"W\n\x0e\x45RC20AssetList\x12"\n\rvega_asset_id\x18\x01 \x01(\tR\x0bvegaAssetId\x12!\n\x0c\x61sset_source\x18\x02 \x01(\tR\x0b\x61ssetSource"6\n\x10\x45RC20AssetDelist\x12"\n\rvega_asset_id\x18\x01 \x01(\tR\x0bvegaAssetId"\xcd\x01\n\x17\x45RC20AssetLimitsUpdated\x12"\n\rvega_asset_id\x18\x01 \x01(\tR\x0bvegaAssetId\x12\x36\n\x17source_ethereum_address\x18\x02 \x01(\tR\x15sourceEthereumAddress\x12\'\n\x0flifetime_limits\x18\x03 \x01(\tR\x0elifetimeLimits\x12-\n\x12withdraw_threshold\x18\x04 \x01(\tR\x11withdrawThreshold"\xaa\x01\n\x0c\x45RC20Deposit\x12"\n\rvega_asset_id\x18\x01 \x01(\tR\x0bvegaAssetId\x12\x36\n\x17source_ethereum_address\x18\x02 \x01(\tR\x15sourceEthereumAddress\x12&\n\x0ftarget_party_id\x18\x03 \x01(\tR\rtargetPartyId\x12\x16\n\x06\x61mount\x18\x04 \x01(\tR\x06\x61mount"\x96\x01\n\x0f\x45RC20Withdrawal\x12"\n\rvega_asset_id\x18\x01 \x01(\tR\x0bvegaAssetId\x12\x36\n\x17target_ethereum_address\x18\x02 \x01(\tR\x15targetEthereumAddress\x12\'\n\x0freference_nonce\x18\x03 \x01(\tR\x0ereferenceNonce"\xcb\x03\n\nERC20Event\x12\x14\n\x05index\x18\x01 \x01(\x04R\x05index\x12\x14\n\x05\x62lock\x18\x02 \x01(\x04R\x05\x62lock\x12\x36\n\nasset_list\x18\xe9\x07 \x01(\x0b\x32\x14.vega.ERC20AssetListH\x00R\tassetList\x12<\n\x0c\x61sset_delist\x18\xea\x07 \x01(\x0b\x32\x16.vega.ERC20AssetDelistH\x00R\x0b\x61ssetDelist\x12/\n\x07\x64\x65posit\x18\xeb\x07 \x01(\x0b\x32\x12.vega.ERC20DepositH\x00R\x07\x64\x65posit\x12\x38\n\nwithdrawal\x18\xec\x07 \x01(\x0b\x32\x15.vega.ERC20WithdrawalH\x00R\nwithdrawal\x12R\n\x14\x61sset_limits_updated\x18\xed\x07 \x01(\x0b\x32\x1d.vega.ERC20AssetLimitsUpdatedH\x00R\x12\x61ssetLimitsUpdated\x12(\n\x0e\x62ridge_stopped\x18\xee\x07 \x01(\x08H\x00R\rbridgeStopped\x12(\n\x0e\x62ridge_resumed\x18\xef\x07 \x01(\x08H\x00R\rbridgeResumedB\x08\n\x06\x61\x63tion"f\n\x10\x45RC20SignerAdded\x12\x1d\n\nnew_signer\x18\x01 \x01(\tR\tnewSigner\x12\x14\n\x05nonce\x18\x02 \x01(\tR\x05nonce\x12\x1d\n\nblock_time\x18\x03 \x01(\x03R\tblockTime"h\n\x12\x45RC20SignerRemoved\x12\x1d\n\nold_signer\x18\x01 \x01(\tR\toldSigner\x12\x14\n\x05nonce\x18\x02 \x01(\tR\x05nonce\x12\x1d\n\nblock_time\x18\x03 \x01(\x03R\tblockTime"m\n\x11\x45RC20ThresholdSet\x12#\n\rnew_threshold\x18\x01 \x01(\rR\x0cnewThreshold\x12\x14\n\x05nonce\x18\x02 \x01(\tR\x05nonce\x12\x1d\n\nblock_time\x18\x03 \x01(\x03R\tblockTime"\x8d\x02\n\x12\x45RC20MultiSigEvent\x12\x14\n\x05index\x18\x01 \x01(\x04R\x05index\x12\x14\n\x05\x62lock\x18\x02 \x01(\x04R\x05\x62lock\x12<\n\x0csigner_added\x18\xe9\x07 \x01(\x0b\x32\x16.vega.ERC20SignerAddedH\x00R\x0bsignerAdded\x12\x42\n\x0esigner_removed\x18\xea\x07 \x01(\x0b\x32\x18.vega.ERC20SignerRemovedH\x00R\rsignerRemoved\x12?\n\rthreshold_set\x18\xeb\x07 \x01(\x0b\x32\x17.vega.ERC20ThresholdSetH\x00R\x0cthresholdSetB\x08\n\x06\x61\x63tion"\x80\x02\n\x0cStakingEvent\x12\x14\n\x05index\x18\x01 \x01(\x04R\x05index\x12\x14\n\x05\x62lock\x18\x02 \x01(\x04R\x05\x62lock\x12@\n\x0fstake_deposited\x18\xe9\x07 \x01(\x0b\x32\x14.vega.StakeDepositedH\x00R\x0estakeDeposited\x12:\n\rstake_removed\x18\xea\x07 \x01(\x0b\x32\x12.vega.StakeRemovedH\x00R\x0cstakeRemoved\x12<\n\x0ctotal_supply\x18\xeb\x07 \x01(\x0b\x32\x16.vega.StakeTotalSupplyH\x00R\x0btotalSupplyB\x08\n\x06\x61\x63tion"\x9a\x01\n\x0eStakeDeposited\x12)\n\x10\x65thereum_address\x18\x01 \x01(\tR\x0f\x65thereumAddress\x12&\n\x0fvega_public_key\x18\x02 \x01(\tR\rvegaPublicKey\x12\x16\n\x06\x61mount\x18\x03 \x01(\tR\x06\x61mount\x12\x1d\n\nblock_time\x18\x04 \x01(\x03R\tblockTime"\x98\x01\n\x0cStakeRemoved\x12)\n\x10\x65thereum_address\x18\x01 \x01(\tR\x0f\x65thereumAddress\x12&\n\x0fvega_public_key\x18\x02 \x01(\tR\rvegaPublicKey\x12\x16\n\x06\x61mount\x18\x03 \x01(\tR\x06\x61mount\x12\x1d\n\nblock_time\x18\x04 \x01(\x03R\tblockTime"Z\n\x10StakeTotalSupply\x12#\n\rtoken_address\x18\x01 \x01(\tR\x0ctokenAddress\x12!\n\x0ctotal_supply\x18\x02 \x01(\tR\x0btotalSupplyB\'Z%code.vegaprotocol.io/vega/protos/vegab\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "vega.chain_events_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z%code.vegaprotocol.io/vega/protos/vega"
    _BUILTINASSETDEPOSIT._serialized_start = 33
    _BUILTINASSETDEPOSIT._serialized_end = 141
    _BUILTINASSETWITHDRAWAL._serialized_start = 143
    _BUILTINASSETWITHDRAWAL._serialized_end = 254
    _BUILTINASSETEVENT._serialized_start = 257
    _BUILTINASSETEVENT._serialized_end = 407
    _ERC20ASSETLIST._serialized_start = 409
    _ERC20ASSETLIST._serialized_end = 496
    _ERC20ASSETDELIST._serialized_start = 498
    _ERC20ASSETDELIST._serialized_end = 552
    _ERC20ASSETLIMITSUPDATED._serialized_start = 555
    _ERC20ASSETLIMITSUPDATED._serialized_end = 760
    _ERC20DEPOSIT._serialized_start = 763
    _ERC20DEPOSIT._serialized_end = 933
    _ERC20WITHDRAWAL._serialized_start = 936
    _ERC20WITHDRAWAL._serialized_end = 1086
    _ERC20EVENT._serialized_start = 1089
    _ERC20EVENT._serialized_end = 1548
    _ERC20SIGNERADDED._serialized_start = 1550
    _ERC20SIGNERADDED._serialized_end = 1652
    _ERC20SIGNERREMOVED._serialized_start = 1654
    _ERC20SIGNERREMOVED._serialized_end = 1758
    _ERC20THRESHOLDSET._serialized_start = 1760
    _ERC20THRESHOLDSET._serialized_end = 1869
    _ERC20MULTISIGEVENT._serialized_start = 1872
    _ERC20MULTISIGEVENT._serialized_end = 2141
    _STAKINGEVENT._serialized_start = 2144
    _STAKINGEVENT._serialized_end = 2400
    _STAKEDEPOSITED._serialized_start = 2403
    _STAKEDEPOSITED._serialized_end = 2557
    _STAKEREMOVED._serialized_start = 2560
    _STAKEREMOVED._serialized_end = 2712
    _STAKETOTALSUPPLY._serialized_start = 2714
    _STAKETOTALSUPPLY._serialized_end = 2804
# @@protoc_insertion_point(module_scope)



import argparse
import logging
from typing import Any, Callable, List, Optional
from vega_sim.environment.agent import Agent
from vega_sim.scenario.ideal_market_maker_v2.scenario import IdealMarketMaker
from vega_sim.scenario.market_crash.price_process import regime_change_random_walk

from vega_sim.scenario.scenario import Scenario

from vega_sim.environment.environment import MarketEnvironmentWithState
from vega_sim.null_service import VegaServiceNull
from vega_sim.scenario.constants import Network
from vega_sim.scenario.ideal_market_maker_v2.agents import (
    MM_WALLET,
    TERMINATE_WALLET,
    TRADER_WALLET,
    BACKGROUND_MARKET,
    AUCTION1_WALLET,
    AUCTION2_WALLET,
    OptimalMarketMaker,
)
from vega_sim.scenario.common.agents import (
    MarketManager,
    MarketOrderTrader,
    MultiRegimeBackgroundMarket,
    MarketRegime,
    OpenAuctionPass,
)

import numpy as np


class MarketCrash(Scenario):
    def __init__(
        self,
        num_steps: int = 120,
        dt: float = 1 / 60 / 24 / 365.25,
        market_decimal: int = 5,
        asset_decimal: int = 5,
        initial_price: float = 100,
        initial_asset_mint: float = 1000,
        sigma_pre: float = 1,
        sigma_post: float = 2,
        drift_pre: float = 0,
        drift_post: float = -1,
        break_point: int = 50,
        kappa: float = 1,
        q_upper: int = 20,
        q_lower: int = -20,
        alpha: float = 10**-4,
        phi: float = 5 * 10**-6,
        spread: float = 0.02,
        block_size: int = 1,
        block_length_seconds: int = 1,
        state_extraction_freq: int = 1,
        position_taker_mint: float = 1000,
        position_taker_buy_intensity: float = 5,
        position_taker_sell_intensity: float = 5,
        noise_buy_intensity: float = 1,
        noise_sell_intensity: float = 1,
        num_position_traders: int = 5,
        num_noise_traders: int = 5,
        step_length_seconds: int = 1,
        settle_at_end: bool = True,
        state_extraction_fn: Optional[
            Callable[[VegaServiceNull, List[Agent]], Any]
        ] = None,
        pause_every_n_steps: Optional[int] = None,
        trim_to_min: Optional[float] = None,
        price_process_fn: Optional[Callable] = None,
    ):
        self.num_steps = num_steps
        self.dt = dt
        self.market_decimal = market_decimal
        self.asset_decimal = asset_decimal
        self.initial_price = initial_price
        self.sigma_pre = sigma_pre
        self.sigma_post = sigma_post
        self.drift_pre = drift_pre
        self.drift_post = drift_post
        self.break_point = break_point
        self.kappa = kappa
        self.q_upper = q_upper
        self.q_lower = q_lower
        self.alpha = alpha
        self.phi = phi
        self.spread = spread
        self.block_size = block_size
        self.block_length_seconds = block_length_seconds
        self.state_extraction_freq = state_extraction_freq
        self.step_length_seconds = step_length_seconds
        self.state_extraction_fn = state_extraction_fn
        self.position_taker_buy_intensity = position_taker_buy_intensity
        self.position_taker_sell_intensity = position_taker_sell_intensity
        self.noise_buy_intensity = noise_buy_intensity
        self.noise_sell_intensity = noise_sell_intensity
        self.pause_every_n_steps = pause_every_n_steps
        self.trim_to_min = trim_to_min
        self.position_taker_mint = position_taker_mint
        self.num_position_traders = num_position_traders
        self.num_noise_traders = num_noise_traders
        self.settle_at_end = settle_at_end
        self.initial_asset_mint = initial_asset_mint
        self.price_process_fn = price_process_fn

    def _generate_price_process(
        self,
        random_state: Optional[np.random.RandomState] = None,
    ):
        return regime_change_random_walk(
            num_steps=self.num_steps + 1,  # Number of steps plus 'initial' state
            sigma_pre=self.sigma_pre,
            sigma_post=self.sigma_post,
            drift_pre=self.drift_pre,
            drift_post=self.drift_post,
            starting_price=self.initial_price,
            break_point=self.break_point,
            decimal_precision=self.market_decimal,
            trim_to_min=self.trim_to_min,
            random_state=random_state,
        )

    def set_up_background_market(
        self,
        vega: VegaServiceNull,
        tag: str = "",
        random_state: Optional[np.random.RandomState] = None,
    ) -> MarketEnvironmentWithState:
        self.market_name = f"BTC:DAI_{tag}"
        self.asset_name = f"tDAI{tag}"

        self.price_process = (
            self.price_process_fn()
            if self.price_process_fn is not None
            else self._generate_price_process()
        )

        market_maker = MarketManager(
            wallet_name=MM_WALLET.name,
            wallet_pass=MM_WALLET.passphrase,
            terminate_wallet_name=TERMINATE_WALLET.name,
            terminate_wallet_pass=TERMINATE_WALLET.passphrase,
            asset_decimal=self.asset_decimal,
            market_decimal=self.market_decimal,
            commitment_amount=60000,
            settlement_price=self.price_process[-1] if self.settle_at_end else None,
            market_name=self.market_name,
            asset_name=self.asset_name,
            tag=str(tag),
        )

        position_traders = []
        noise_traders = []

        for i in range(self.num_noise_traders):
            noise_traders.append(
                MarketOrderTrader(
                    wallet_name=TRADER_WALLET.name,
                    wallet_pass=TRADER_WALLET.passphrase,
                    market_name=self.market_name,
                    asset_name=self.asset_name,
                    tag=f"{tag}_noise_{i}",
                    initial_asset_mint=self.position_taker_mint,
                    buy_intensity=self.noise_buy_intensity,
                    sell_intensity=self.noise_sell_intensity,
                    random_state=random_state,
                )
            )
        for i in range(self.num_position_traders):
            position_traders.append(
                MarketOrderTrader(
                    wallet_name=TRADER_WALLET.name,
                    wallet_pass=TRADER_WALLET.passphrase,
                    market_name=self.market_name,
                    asset_name=self.asset_name,
                    initial_asset_mint=self.position_taker_mint,
                    tag=f"{tag}_pos_{i}",
                    buy_intensity=self.position_taker_buy_intensity,
                    sell_intensity=self.position_taker_sell_intensity,
                    random_state=random_state,
                )
            )

        background_market = MultiRegimeBackgroundMarket(
            wallet_name=BACKGROUND_MARKET.name,
            wallet_pass=BACKGROUND_MARKET.passphrase,
            market_name=self.market_name,
            asset_name=self.asset_name,
            market_regimes=[
                MarketRegime(
                    spread=self.spread,
                    tick_spacing=0.1,
                    num_levels_per_side=25,
                    base_volume_size=1,
                    order_distribution_buy_kappa=self.kappa,
                    order_distribution_sell_kappa=self.kappa,
                    from_timepoint=0,
                    thru_timepoint=10000,
                )
            ],
            price_process=self.price_process,
            tag=str(tag),
        )

        auctionpass1 = OpenAuctionPass(
            wallet_name=AUCTION1_WALLET.name,
            wallet_pass=AUCTION1_WALLET.passphrase,
            side="SIDE_BUY",
            initial_price=self.initial_price,
            market_name=self.market_name,
            asset_name=self.asset_name,
            initial_asset_mint=self.initial_asset_mint,
            tag=str(tag),
        )

        auctionpass2 = OpenAuctionPass(
            wallet_name=AUCTION2_WALLET.name,
            wallet_pass=AUCTION2_WALLET.passphrase,
            side="SIDE_SELL",
            initial_price=self.initial_price,
            market_name=self.market_name,
            asset_name=self.asset_name,
            initial_asset_mint=self.initial_asset_mint,
            tag=str(tag),
        )

        env = MarketEnvironmentWithState(
            agents=[
                market_maker,
                background_market,
                auctionpass1,
                auctionpass2,
            ]
            + noise_traders
            + position_traders,
            n_steps=self.num_steps,
            transactions_per_block=self.block_size,
            vega_service=vega,
            state_extraction_freq=self.state_extraction_freq,
            step_length_seconds=self.step_length_seconds,
            block_length_seconds=self.block_length_seconds,
            state_extraction_fn=self.state_extraction_fn,
            pause_every_n_steps=self.pause_every_n_steps,
        )
        return env

    def run_iteration(
        self,
        vega: VegaServiceNull,
        network: Optional[Network] = None,
        pause_at_completion: bool = False,
        tag: Optional[str] = None,
        random_state: Optional[np.random.RandomState] = None,
    ):
        env = self.set_up_background_market(
            vega=vega, tag=tag if tag is not None else str(0), random_state=random_state
        )
        result = env.run(
            pause_at_completion=pause_at_completion,
        )
        return result


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--debug", action="store_true")
    args = parser.parse_args()

    logging.basicConfig(level=logging.INFO if not args.debug else logging.DEBUG)

    step_length = 60 * 60

    scenario = MarketCrash(
        num_steps=200,
        sigma_pre=1,
        sigma_post=2,
        drift_pre=0.1,
        drift_post=-0.2,
        break_point=800,
        initial_price=10,
        # step_length_seconds=60,
    )

    with VegaServiceNull(
        warn_on_raw_data_access=False,
        run_with_console=True,
        seconds_per_block=40,  # Heuristic
    ) as vega:
        scenario.run_iteration(vega=vega, pause_at_completion=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vega/data/v1/data.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x17vega/data/v1/data.proto\x12\x0cvega.data.v1"&\n\nETHAddress\x12\x18\n\x07\x61\x64\x64ress\x18\x01 \x01(\tR\x07\x61\x64\x64ress"\x1a\n\x06PubKey\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key"\x80\x01\n\x06Signer\x12/\n\x07pub_key\x18\x01 \x01(\x0b\x32\x14.vega.data.v1.PubKeyH\x00R\x06pubKey\x12;\n\x0b\x65th_address\x18\x02 \x01(\x0b\x32\x18.vega.data.v1.ETHAddressH\x00R\nethAddressB\x08\n\x06signer"4\n\x08Property\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value"\xaf\x01\n\x04\x44\x61ta\x12.\n\x07signers\x18\x01 \x03(\x0b\x32\x14.vega.data.v1.SignerR\x07signers\x12*\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x16.vega.data.v1.PropertyR\x04\x64\x61ta\x12(\n\x10matched_spec_ids\x18\x03 \x03(\tR\x0ematchedSpecIds\x12!\n\x0c\x62roadcast_at\x18\x04 \x01(\x03R\x0b\x62roadcastAt"6\n\x0c\x45xternalData\x12&\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x12.vega.data.v1.DataR\x04\x64\x61taB/Z-code.vegaprotocol.io/vega/protos/vega/data/v1b\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "vega.data.v1.data_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z-code.vegaprotocol.io/vega/protos/vega/data/v1"
    _ETHADDRESS._serialized_start = 41
    _ETHADDRESS._serialized_end = 79
    _PUBKEY._serialized_start = 81
    _PUBKEY._serialized_end = 107
    _SIGNER._serialized_start = 110
    _SIGNER._serialized_end = 238
    _PROPERTY._serialized_start = 240
    _PROPERTY._serialized_end = 292
    _DATA._serialized_start = 295
    _DATA._serialized_end = 470
    _EXTERNALDATA._serialized_start = 472
    _EXTERNALDATA._serialized_end = 526
# @@protoc_insertion_point(module_scope)



import argparse
import logging

import vega_sim.parameter_test.parameter.experiment as experiment
from vega_sim.parameter_test.parameter.configs import CONFIGS


if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument("-c", "--config")

    args = parser.parse_args()
    logging.basicConfig(level=logging.INFO)
    config_map = {c.name: c for c in CONFIGS}

    experiment_to_run = config_map[args.config]
    results = experiment.run_single_parameter_experiment(experiment_to_run)
    experiment.output_logs(
        results, experiment_to_run.parameter_to_vary, experiment=experiment_to_run
    )
